from __future__ import annotations

from dataclasses import dataclass
from typing import Optional

from leeger.enum.MatchupType import MatchupType
from leeger.exception import InvalidMatchupFormatException
from leeger.model.abstract.EqualityCheck import EqualityCheck
from leeger.model.abstract.UniqueId import UniqueId
from leeger.util.CustomLogger import CustomLogger
from leeger.util.JSONSerializable import JSONSerializable
from leeger.util.equality import modelEquals


@dataclass(kw_only=True, eq=False)
class Performance(UniqueId, EqualityCheck, JSONSerializable):
    __LOGGER = CustomLogger.getLogger()
    teamId: str
    teamScore: float | int
    hasTiebreaker: bool = False
    matchupType: MatchupType = MatchupType.REGULAR_SEASON
    # This is used to link matchups that span over multiple weeks
    multiWeekMatchupId: Optional[str] = None

    def equals(
        self,
        otherPerformance: Performance,
        *,
        ignoreIds: bool = False,
        ignoreBaseIds: bool = False,
        logDifferences: bool = False,
    ) -> bool:
        """
        Checks if *this* Performance is the same as the given Performance.
        """

        return modelEquals(
            objA=self,
            objB=otherPerformance,
            baseFields={"teamId", "teamScore", "hasTiebreaker", "matchupType"},
            idFields={"multiWeekMatchupId"},
            parentKey="Performance",
            ignoreIdFields=ignoreIds,
            ignoreBaseIdField=ignoreBaseIds,
            logDifferences=logDifferences,
        )

    def __eq__(self, otherPerformance: Performance) -> bool:
        self.__LOGGER.info("Use .equals() for more options when comparing Performance instances.")
        return self.equals(otherPerformance=otherPerformance)

    def __add__(self, otherPerformance: Performance):
        """
        Adds 2 Performances together.

        Returns:
            leeger.model.league.Matchup
        """
        from leeger.model.league import Matchup
        from leeger.validate import matchupValidation

        if self.matchupType != otherPerformance.matchupType:
            raise InvalidMatchupFormatException(
                f"Cannot make a matchup from conflicting matchup types '{self.matchupType}' and '{otherPerformance.matchupType}'."
            )
        if self.multiWeekMatchupId != otherPerformance.multiWeekMatchupId:
            raise InvalidMatchupFormatException(
                f"Cannot make a matchup from conflicting multi-week matchup IDs '{self.multiWeekMatchupId}' and '{otherPerformance.multiWeekMatchupId}'."
            )
        tiebreakerInfoLost = list()
        if self.hasTiebreaker:
            tiebreakerInfoLost.append(f"Performance {self.id} had tiebreaker")
        if otherPerformance.hasTiebreaker:
            tiebreakerInfoLost.append(f"Performance {otherPerformance.id} had tiebreaker")
        if tiebreakerInfoLost:
            self.__LOGGER.warning(
                f"Combining performances caused loss of tiebreakers: {tiebreakerInfoLost}."
            )
        matchup = Matchup(
            teamAId=self.teamId,
            teamBId=otherPerformance.teamId,
            teamAScore=self.teamScore,
            teamBScore=otherPerformance.teamScore,
            matchupType=self.matchupType,
            multiWeekMatchupId=self.multiWeekMatchupId,
        )
        # validate new matchup
        matchupValidation.runAllChecks(matchup)
        return matchup

    def toJson(self) -> dict:
        return {
            "id": self.id,
            "teamId": self.teamId,
            "teamScore": self.teamScore,
            "hasTiebreaker": self.hasTiebreaker,
            "matchupType": self.matchupType.name,
            "multiWeekMatchupId": self.multiWeekMatchupId,
        }



import configparser
import os
from typing import Optional


class ConfigReader:
    """
    Used to read from .properties files
    """

    @staticmethod
    def get(
        section: str, name: str, *, asType: str | list = str, propFile: str = "app.properties"
    ) -> Optional[str | int | float | bool]:
        configParser = configparser.ConfigParser(
            converters={"list": lambda x: [i.strip() for i in x.split(",")]}
        )
        propertiesDirectory = os.path.abspath(
            os.path.join(os.path.dirname(os.path.realpath(__file__)), "../properties")
        )
        configParser.read(os.path.join(propertiesDirectory, propFile))
        value = None
        # cast as type
        if asType == list:
            value = configParser.getlist(section, name)
        elif asType == str:
            value = configParser[section][name]
        else:
            raise ValueError(f"Type '{asType}' not supported for conversion.")
        return value



import unittest
from unittest.mock import patch, Mock

from leeger.enum.MatchupType import MatchupType

from leeger.league_loader.ESPNLeagueLoader import ESPNLeagueLoader
from leeger.model.league.Division import Division
from leeger.model.league.League import League
from leeger.model.league.Matchup import Matchup
from leeger.model.league.Owner import Owner
from leeger.model.league.Team import Team
from leeger.model.league.Week import Week
from leeger.model.league.Year import Year


class TestESPNLeagueLoader(unittest.TestCase):
    def test_loadLeague_nonIntPassingStringForLeagueId(self):
        with self.assertRaises(ValueError) as context:
            ESPNLeagueLoader("a", [2000])
        self.assertEqual("League ID 'a' could not be turned into an int.", str(context.exception))

    @patch("espn_api.football.League")
    def test_load_league_happyPath(self, mockLeague):
        # mock first year (2022)
        mockEspnLeague2022 = Mock()
        mockEspnLeague2022.year = 2022
        mockEspnLeague2022.current_week = 4
        mockEspnLeague2022.settings.name = "Test League 2022"
        mockEspnLeague2022.settings.reg_season_count = 2
        mockEspnLeague2022.settings.playoff_team_count = 3
        mockEspnLeague2022.settings.division_map = {0: "d1_2022", 1: "d2_2022"}
        mockTeam1_2022 = Mock(
            team_id=1,
            owner="Owner 1",
            team_name="Team 1",
            outcomes=["W", "W", "U", "W"],
            scores=[100, 110, 0, 100],
            standing=1,
            division_id=0,
        )
        mockTeam2_2022 = Mock(
            team_id=2,
            owner="Owner 2",
            team_name="Team 2",
            outcomes=["L", "L", "L", "L"],
            scores=[100, 70, 1, 1],
            standing=7,
            division_id=0,
        )
        mockTeam3_2022 = Mock(
            team_id=3,
            owner="Owner 3",
            team_name="Team 3",
            outcomes=["W", "L", "L", "W"],
            scores=[90.5, 100, 80, 2],
            standing=3,
            division_id=0,
        )
        mockTeam4_2022 = Mock(
            team_id=4,
            owner="Owner 4",
            team_name="Team 4",
            outcomes=["L", "W", "U", "L"],
            scores=[70.5, 80, 0, 1],
            standing=5,
            division_id=0,
        )
        mockTeam5_2022 = Mock(
            team_id=5,
            owner="Owner 5",
            team_name="Team 5",
            outcomes=["W", "L", "W", "W"],
            scores=[110, 80, 2, 2],
            standing=4,
            division_id=1,
        )
        mockTeam6_2022 = Mock(
            team_id=6,
            owner="Owner 6",
            team_name="Team 6",
            outcomes=["L", "W", "W", "W"],
            scores=[60, 90, 2, 2],
            standing=6,
            division_id=1,
        )
        mockTeam7_2022 = Mock(
            team_id=7,
            owner="Owner 7",
            team_name="Team 7",
            outcomes=["W", "W", "W", "L"],
            scores=[120, 130, 90, 90],
            standing=2,
            division_id=1,
        )
        mockTeam8_2022 = Mock(
            team_id=8,
            owner="Owner 8",
            team_name="Team 8",
            outcomes=["L", "L", "L", "L"],
            scores=[50, 40, 1, 1],
            standing=8,
            division_id=1,
        )
        # playoff seeds:
        # Team1: 1
        # Team7: 2
        # Team3: 3
        mockTeam1_2022.schedule = [mockTeam2_2022, mockTeam3_2022, mockTeam4_2022, mockTeam7_2022]
        mockTeam2_2022.schedule = [mockTeam1_2022, mockTeam4_2022, mockTeam5_2022, mockTeam3_2022]
        mockTeam3_2022.schedule = [mockTeam4_2022, mockTeam1_2022, mockTeam7_2022, mockTeam2_2022]
        mockTeam4_2022.schedule = [mockTeam3_2022, mockTeam2_2022, mockTeam1_2022, mockTeam5_2022]
        mockTeam5_2022.schedule = [mockTeam6_2022, mockTeam7_2022, mockTeam2_2022, mockTeam4_2022]
        mockTeam6_2022.schedule = [mockTeam5_2022, mockTeam8_2022, mockTeam8_2022, mockTeam8_2022]
        mockTeam7_2022.schedule = [mockTeam8_2022, mockTeam5_2022, mockTeam3_2022, mockTeam1_2022]
        mockTeam8_2022.schedule = [mockTeam7_2022, mockTeam6_2022, mockTeam6_2022, mockTeam6_2022]
        mockEspnLeague2022.teams = [
            mockTeam1_2022,
            mockTeam2_2022,
            mockTeam3_2022,
            mockTeam4_2022,
            mockTeam5_2022,
            mockTeam6_2022,
            mockTeam7_2022,
            mockTeam8_2022,
        ]

        # mock second year (2023)
        mockEspnLeague2023 = Mock()
        mockEspnLeague2023.year = 2023
        mockEspnLeague2023.current_week = 1
        mockEspnLeague2023.settings.name = "Test League 2023"
        mockEspnLeague2023.settings.reg_season_count = 1
        mockEspnLeague2023.settings.division_map = {0: "d1_2023", 1: "d2_2023"}
        mockTeam1_2023 = Mock(
            team_id=1,
            owner="Owner 1",
            team_name="Team 1",
            outcomes=["W"],
            scores=[100],
            division_id=0,
        )
        mockTeam2_2023 = Mock(
            team_id=2,
            owner="Owner 2",
            team_name="Team 2",
            outcomes=["L"],
            scores=[100],
            division_id=0,
        )
        mockTeam3_2023 = Mock(
            team_id=3,
            owner="Owner 3",
            team_name="Team 3",
            outcomes=["W"],
            scores=[90.5],
            division_id=0,
        )
        mockTeam4_2023 = Mock(
            team_id=4,
            owner="Owner 4",
            team_name="Team 4",
            outcomes=["L"],
            scores=[70.5],
            division_id=0,
        )
        mockTeam5_2023 = Mock(
            team_id=5,
            owner="Owner 5",
            team_name="Team 5",
            outcomes=["W"],
            scores=[110],
            division_id=1,
        )
        mockTeam6_2023 = Mock(
            team_id=6,
            owner="Owner 6",
            team_name="Team 6",
            outcomes=["L"],
            scores=[60],
            division_id=1,
        )
        mockTeam7_2023 = Mock(
            team_id=7,
            owner="Owner 7",
            team_name="Team 7",
            outcomes=["W"],
            scores=[120],
            division_id=1,
        )
        mockTeam8_2023 = Mock(
            team_id=8,
            owner="Owner 8",
            team_name="Team 8",
            outcomes=["L"],
            scores=[50],
            division_id=1,
        )
        mockTeam1_2023.schedule = [mockTeam2_2023]
        mockTeam2_2023.schedule = [mockTeam1_2023]
        mockTeam3_2023.schedule = [mockTeam4_2023]
        mockTeam4_2023.schedule = [mockTeam3_2023]
        mockTeam5_2023.schedule = [mockTeam6_2023]
        mockTeam6_2023.schedule = [mockTeam5_2023]
        mockTeam7_2023.schedule = [mockTeam8_2023]
        mockTeam8_2023.schedule = [mockTeam7_2023]
        mockEspnLeague2023.teams = [
            mockTeam1_2023,
            mockTeam2_2023,
            mockTeam3_2023,
            mockTeam4_2023,
            mockTeam5_2023,
            mockTeam6_2023,
            mockTeam7_2023,
            mockTeam8_2023,
        ]

        mockLeague.side_effect = [mockEspnLeague2022, mockEspnLeague2023]

        loader = ESPNLeagueLoader("123", [2022, 2023])
        league = loader.loadLeague()

        # expected league

        owner1 = Owner(name="Owner 1")
        owner2 = Owner(name="Owner 2")
        owner3 = Owner(name="Owner 3")
        owner4 = Owner(name="Owner 4")
        owner5 = Owner(name="Owner 5")
        owner6 = Owner(name="Owner 6")
        owner7 = Owner(name="Owner 7")
        owner8 = Owner(name="Owner 8")

        division1_2022 = Division(name="d1_2022")
        division2_2022 = Division(name="d2_2022")

        division1_2023 = Division(name="d1_2023")
        division2_2023 = Division(name="d2_2023")

        team1_2022 = Team(ownerId=owner1.id, name="Team 1", divisionId=division1_2022.id)
        team2_2022 = Team(ownerId=owner2.id, name="Team 2", divisionId=division1_2022.id)
        team3_2022 = Team(ownerId=owner3.id, name="Team 3", divisionId=division1_2022.id)
        team4_2022 = Team(ownerId=owner4.id, name="Team 4", divisionId=division1_2022.id)
        team5_2022 = Team(ownerId=owner5.id, name="Team 5", divisionId=division2_2022.id)
        team6_2022 = Team(ownerId=owner6.id, name="Team 6", divisionId=division2_2022.id)
        team7_2022 = Team(ownerId=owner7.id, name="Team 7", divisionId=division2_2022.id)
        team8_2022 = Team(ownerId=owner8.id, name="Team 8", divisionId=division2_2022.id)

        team1_2023 = Team(ownerId=owner1.id, name="Team 1", divisionId=division1_2023.id)
        team2_2023 = Team(ownerId=owner2.id, name="Team 2", divisionId=division1_2023.id)
        team3_2023 = Team(ownerId=owner3.id, name="Team 3", divisionId=division1_2023.id)
        team4_2023 = Team(ownerId=owner4.id, name="Team 4", divisionId=division1_2023.id)
        team5_2023 = Team(ownerId=owner5.id, name="Team 5", divisionId=division2_2023.id)
        team6_2023 = Team(ownerId=owner6.id, name="Team 6", divisionId=division2_2023.id)
        team7_2023 = Team(ownerId=owner7.id, name="Team 7", divisionId=division2_2023.id)
        team8_2023 = Team(ownerId=owner8.id, name="Team 8", divisionId=division2_2023.id)

        expectedLeague = League(
            name="Test League 2023",
            owners=[owner1, owner2, owner3, owner4, owner5, owner6, owner7, owner8],
            years=[
                Year(
                    yearNumber=2022,
                    teams=[
                        team1_2022,
                        team2_2022,
                        team3_2022,
                        team4_2022,
                        team5_2022,
                        team6_2022,
                        team7_2022,
                        team8_2022,
                    ],
                    weeks=[
                        Week(
                            weekNumber=1,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team2_2022.id,
                                    teamAScore=100,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2022.id,
                                    teamBId=team4_2022.id,
                                    teamAScore=90.5,
                                    teamBScore=70.5,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2022.id,
                                    teamBId=team6_2022.id,
                                    teamAScore=110,
                                    teamBScore=60,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team7_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=120,
                                    teamBScore=50,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=2,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team3_2022.id,
                                    teamAScore=110,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team2_2022.id,
                                    teamBId=team4_2022.id,
                                    teamAScore=70,
                                    teamBScore=80,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2022.id,
                                    teamBId=team7_2022.id,
                                    teamAScore=80,
                                    teamBScore=130,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team6_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=90,
                                    teamBScore=40,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=3,
                            matchups=[
                                Matchup(
                                    teamAId=team2_2022.id,
                                    teamBId=team5_2022.id,
                                    teamAScore=1,
                                    teamBScore=2,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2022.id,
                                    teamBId=team7_2022.id,
                                    teamAScore=80,
                                    teamBScore=90,
                                    matchupType=MatchupType.PLAYOFF,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team6_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=2,
                                    teamBScore=1,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=4,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team7_2022.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.CHAMPIONSHIP,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team2_2022.id,
                                    teamBId=team3_2022.id,
                                    teamAScore=1,
                                    teamBScore=2,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team4_2022.id,
                                    teamBId=team5_2022.id,
                                    teamAScore=1,
                                    teamBScore=2,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team6_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=2,
                                    teamBScore=1,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                    ],
                    yearSettings=None,
                    divisions=[division1_2022, division2_2022],
                ),
                Year(
                    yearNumber=2023,
                    teams=[
                        team1_2023,
                        team2_2023,
                        team3_2023,
                        team4_2023,
                        team5_2023,
                        team6_2023,
                        team7_2023,
                        team8_2023,
                    ],
                    weeks=[
                        Week(
                            weekNumber=1,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2023.id,
                                    teamBId=team2_2023.id,
                                    teamAScore=100,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2023.id,
                                    teamBId=team4_2023.id,
                                    teamAScore=90.5,
                                    teamBScore=70.5,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2023.id,
                                    teamBId=team6_2023.id,
                                    teamAScore=110,
                                    teamBScore=60,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team7_2023.id,
                                    teamBId=team8_2023.id,
                                    teamAScore=120,
                                    teamBScore=50,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        )
                    ],
                    yearSettings=None,
                    divisions=[division1_2023, division2_2023],
                ),
            ],
        )
        self.assertTrue(league.equals(expectedLeague, ignoreBaseIds=True, ignoreIds=True))
        # check multiWeekMatchupIds
        for year in league.years:
            for week in year.weeks:
                for matchup in week.matchups:
                    self.assertIsNone(matchup.multiWeekMatchupId)

    @patch("espn_api.football.League")
    def test_load_league_happyPath_withOwnerNamesAndAliases(self, mockLeague):
        # mock first year (2022)
        mockEspnLeague2022 = Mock()
        mockEspnLeague2022.year = 2022
        mockEspnLeague2022.current_week = 4
        mockEspnLeague2022.settings.name = "Test League 2022"
        mockEspnLeague2022.settings.reg_season_count = 2
        mockEspnLeague2022.settings.playoff_team_count = 3
        mockEspnLeague2022.settings.division_map = {0: "d1_2022", 1: "d2_2022"}
        mockTeam1_2022 = Mock(
            team_id=1,
            owner="Owner 1",
            team_name="Team 1",
            outcomes=["W", "W", "U", "W"],
            scores=[100, 110, 0, 100],
            standing=1,
            division_id=0,
        )
        mockTeam2_2022 = Mock(
            team_id=2,
            owner="Owner 2",
            team_name="Team 2",
            outcomes=["L", "L", "L", "L"],
            scores=[100, 70, 1, 1],
            standing=7,
            division_id=0,
        )
        mockTeam3_2022 = Mock(
            team_id=3,
            owner="Owner 3",
            team_name="Team 3",
            outcomes=["W", "L", "L", "W"],
            scores=[90.5, 100, 80, 2],
            standing=3,
            division_id=0,
        )
        mockTeam4_2022 = Mock(
            team_id=4,
            owner="Owner 4",
            team_name="Team 4",
            outcomes=["L", "W", "U", "L"],
            scores=[70.5, 80, 0, 1],
            standing=5,
            division_id=0,
        )
        mockTeam5_2022 = Mock(
            team_id=5,
            owner="Owner 5",
            team_name="Team 5",
            outcomes=["W", "L", "W", "W"],
            scores=[110, 80, 2, 2],
            standing=4,
            division_id=1,
        )
        mockTeam6_2022 = Mock(
            team_id=6,
            owner="Owner 6",
            team_name="Team 6",
            outcomes=["L", "W", "W", "W"],
            scores=[60, 90, 2, 2],
            standing=6,
            division_id=1,
        )
        mockTeam7_2022 = Mock(
            team_id=7,
            owner="Owner 7",
            team_name="Team 7",
            outcomes=["W", "W", "W", "L"],
            scores=[120, 130, 90, 90],
            standing=2,
            division_id=1,
        )
        mockTeam8_2022 = Mock(
            team_id=8,
            owner="Owner 8",
            team_name="Team 8",
            outcomes=["L", "L", "L", "L"],
            scores=[50, 40, 1, 1],
            standing=8,
            division_id=1,
        )
        # playoff seeds:
        # Team1: 1
        # Team7: 2
        # Team3: 3
        mockTeam1_2022.schedule = [mockTeam2_2022, mockTeam3_2022, mockTeam4_2022, mockTeam7_2022]
        mockTeam2_2022.schedule = [mockTeam1_2022, mockTeam4_2022, mockTeam5_2022, mockTeam3_2022]
        mockTeam3_2022.schedule = [mockTeam4_2022, mockTeam1_2022, mockTeam7_2022, mockTeam2_2022]
        mockTeam4_2022.schedule = [mockTeam3_2022, mockTeam2_2022, mockTeam1_2022, mockTeam5_2022]
        mockTeam5_2022.schedule = [mockTeam6_2022, mockTeam7_2022, mockTeam2_2022, mockTeam4_2022]
        mockTeam6_2022.schedule = [mockTeam5_2022, mockTeam8_2022, mockTeam8_2022, mockTeam8_2022]
        mockTeam7_2022.schedule = [mockTeam8_2022, mockTeam5_2022, mockTeam3_2022, mockTeam1_2022]
        mockTeam8_2022.schedule = [mockTeam7_2022, mockTeam6_2022, mockTeam6_2022, mockTeam6_2022]
        mockEspnLeague2022.teams = [
            mockTeam1_2022,
            mockTeam2_2022,
            mockTeam3_2022,
            mockTeam4_2022,
            mockTeam5_2022,
            mockTeam6_2022,
            mockTeam7_2022,
            mockTeam8_2022,
        ]

        # mock second year (2023)
        mockEspnLeague2023 = Mock()
        mockEspnLeague2023.year = 2023
        mockEspnLeague2023.current_week = 1
        mockEspnLeague2023.settings.name = "Test League 2023"
        mockEspnLeague2023.settings.reg_season_count = 1
        mockEspnLeague2023.settings.division_map = {0: "d1_2023", 1: "d2_2023"}
        mockTeam1_2023 = Mock(
            team_id=1,
            owner="Owner 1",
            team_name="Team 1",
            outcomes=["W"],
            scores=[100],
            division_id=0,
        )
        mockTeam2_2023 = Mock(
            team_id=2,
            owner="Owner 2",
            team_name="Team 2",
            outcomes=["L"],
            scores=[100],
            division_id=0,
        )
        mockTeam3_2023 = Mock(
            team_id=3,
            owner="Owner 3",
            team_name="Team 3",
            outcomes=["W"],
            scores=[90.5],
            division_id=0,
        )
        mockTeam4_2023 = Mock(
            team_id=4,
            owner="Owner 4",
            team_name="Team 4",
            outcomes=["L"],
            scores=[70.5],
            division_id=0,
        )
        mockTeam5_2023 = Mock(
            team_id=5,
            owner="Owner 5",
            team_name="Team 5",
            outcomes=["W"],
            scores=[110],
            division_id=1,
        )
        mockTeam6_2023 = Mock(
            team_id=6,
            owner="Owner 6",
            team_name="Team 6",
            outcomes=["L"],
            scores=[60],
            division_id=1,
        )
        mockTeam7_2023 = Mock(
            team_id=7,
            owner="Owner 7",
            team_name="Team 7",
            outcomes=["W"],
            scores=[120],
            division_id=1,
        )
        mockTeam8_2023 = Mock(
            team_id=8,
            owner="Owner 8",
            team_name="Team 8",
            outcomes=["L"],
            scores=[50],
            division_id=1,
        )
        mockTeam1_2023.schedule = [mockTeam2_2023]
        mockTeam2_2023.schedule = [mockTeam1_2023]
        mockTeam3_2023.schedule = [mockTeam4_2023]
        mockTeam4_2023.schedule = [mockTeam3_2023]
        mockTeam5_2023.schedule = [mockTeam6_2023]
        mockTeam6_2023.schedule = [mockTeam5_2023]
        mockTeam7_2023.schedule = [mockTeam8_2023]
        mockTeam8_2023.schedule = [mockTeam7_2023]
        mockEspnLeague2023.teams = [
            mockTeam1_2023,
            mockTeam2_2023,
            mockTeam3_2023,
            mockTeam4_2023,
            mockTeam5_2023,
            mockTeam6_2023,
            mockTeam7_2023,
            mockTeam8_2023,
        ]

        mockLeague.side_effect = [mockEspnLeague2022, mockEspnLeague2023]

        loader = ESPNLeagueLoader(
            "123",
            [2022, 2023],
            ownerNamesAndAliases={
                "o1": ["Owner 1"],
                "o2": ["Owner 2"],
                "o3": ["Owner 3"],
                "o4": ["Owner 4"],
                "o5": ["Owner 5"],
                "o6": ["Owner 6"],
                "o7": ["Owner 7"],
                "o8": ["Owner 8"],
            },
        )
        league = loader.loadLeague()

        # expected league

        owner1 = Owner(name="o1")
        owner2 = Owner(name="o2")
        owner3 = Owner(name="o3")
        owner4 = Owner(name="o4")
        owner5 = Owner(name="o5")
        owner6 = Owner(name="o6")
        owner7 = Owner(name="o7")
        owner8 = Owner(name="o8")

        division1_2022 = Division(name="d1_2022")
        division2_2022 = Division(name="d2_2022")

        division1_2023 = Division(name="d1_2023")
        division2_2023 = Division(name="d2_2023")

        team1_2022 = Team(ownerId=owner1.id, name="Team 1", divisionId=division1_2022.id)
        team2_2022 = Team(ownerId=owner2.id, name="Team 2", divisionId=division1_2022.id)
        team3_2022 = Team(ownerId=owner3.id, name="Team 3", divisionId=division1_2022.id)
        team4_2022 = Team(ownerId=owner4.id, name="Team 4", divisionId=division1_2022.id)
        team5_2022 = Team(ownerId=owner5.id, name="Team 5", divisionId=division2_2022.id)
        team6_2022 = Team(ownerId=owner6.id, name="Team 6", divisionId=division2_2022.id)
        team7_2022 = Team(ownerId=owner7.id, name="Team 7", divisionId=division2_2022.id)
        team8_2022 = Team(ownerId=owner8.id, name="Team 8", divisionId=division2_2022.id)

        team1_2023 = Team(ownerId=owner1.id, name="Team 1", divisionId=division1_2023.id)
        team2_2023 = Team(ownerId=owner2.id, name="Team 2", divisionId=division1_2023.id)
        team3_2023 = Team(ownerId=owner3.id, name="Team 3", divisionId=division1_2023.id)
        team4_2023 = Team(ownerId=owner4.id, name="Team 4", divisionId=division1_2023.id)
        team5_2023 = Team(ownerId=owner5.id, name="Team 5", divisionId=division2_2023.id)
        team6_2023 = Team(ownerId=owner6.id, name="Team 6", divisionId=division2_2023.id)
        team7_2023 = Team(ownerId=owner7.id, name="Team 7", divisionId=division2_2023.id)
        team8_2023 = Team(ownerId=owner8.id, name="Team 8", divisionId=division2_2023.id)

        expectedLeague = League(
            name="Test League 2023",
            owners=[owner1, owner2, owner3, owner4, owner5, owner6, owner7, owner8],
            years=[
                Year(
                    yearNumber=2022,
                    teams=[
                        team1_2022,
                        team2_2022,
                        team3_2022,
                        team4_2022,
                        team5_2022,
                        team6_2022,
                        team7_2022,
                        team8_2022,
                    ],
                    weeks=[
                        Week(
                            weekNumber=1,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team2_2022.id,
                                    teamAScore=100,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2022.id,
                                    teamBId=team4_2022.id,
                                    teamAScore=90.5,
                                    teamBScore=70.5,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2022.id,
                                    teamBId=team6_2022.id,
                                    teamAScore=110,
                                    teamBScore=60,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team7_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=120,
                                    teamBScore=50,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=2,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team3_2022.id,
                                    teamAScore=110,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team2_2022.id,
                                    teamBId=team4_2022.id,
                                    teamAScore=70,
                                    teamBScore=80,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2022.id,
                                    teamBId=team7_2022.id,
                                    teamAScore=80,
                                    teamBScore=130,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team6_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=90,
                                    teamBScore=40,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=3,
                            matchups=[
                                Matchup(
                                    teamAId=team2_2022.id,
                                    teamBId=team5_2022.id,
                                    teamAScore=1,
                                    teamBScore=2,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2022.id,
                                    teamBId=team7_2022.id,
                                    teamAScore=80,
                                    teamBScore=90,
                                    matchupType=MatchupType.PLAYOFF,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team6_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=2,
                                    teamBScore=1,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=4,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team7_2022.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.CHAMPIONSHIP,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team2_2022.id,
                                    teamBId=team3_2022.id,
                                    teamAScore=1,
                                    teamBScore=2,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team4_2022.id,
                                    teamBId=team5_2022.id,
                                    teamAScore=1,
                                    teamBScore=2,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team6_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=2,
                                    teamBScore=1,
                                    matchupType=MatchupType.IGNORE,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                    ],
                    yearSettings=None,
                    divisions=[division1_2022, division2_2022],
                ),
                Year(
                    yearNumber=2023,
                    teams=[
                        team1_2023,
                        team2_2023,
                        team3_2023,
                        team4_2023,
                        team5_2023,
                        team6_2023,
                        team7_2023,
                        team8_2023,
                    ],
                    weeks=[
                        Week(
                            weekNumber=1,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2023.id,
                                    teamBId=team2_2023.id,
                                    teamAScore=100,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2023.id,
                                    teamBId=team4_2023.id,
                                    teamAScore=90.5,
                                    teamBScore=70.5,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2023.id,
                                    teamBId=team6_2023.id,
                                    teamAScore=110,
                                    teamBScore=60,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team7_2023.id,
                                    teamBId=team8_2023.id,
                                    teamAScore=120,
                                    teamBScore=50,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=False,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        )
                    ],
                    yearSettings=None,
                    divisions=[division1_2023, division2_2023],
                ),
            ],
        )
        self.assertTrue(league.equals(expectedLeague, ignoreBaseIds=True, ignoreIds=True))
        # check multiWeekMatchupIds
        for year in league.years:
            for week in year.weeks:
                for matchup in week.matchups:
                    self.assertIsNone(matchup.multiWeekMatchupId)

    @patch("espn_api.football.League")
    def test_load_league_withLeagueName(self, mockLeague):
        # mock first year (2022)
        mockEspnLeague2022 = Mock()
        mockEspnLeague2022.year = 2022
        mockEspnLeague2022.current_week = 4
        mockEspnLeague2022.settings.name = "Test League 2022"
        mockEspnLeague2022.settings.reg_season_count = 2
        mockEspnLeague2022.settings.playoff_team_count = 3
        mockEspnLeague2022.settings.division_map = {0: "d1_2022", 1: "d2_2022"}
        mockTeam1_2022 = Mock(
            team_id=1,
            owner="Owner 1",
            team_name="Team 1",
            outcomes=["W", "W", "U", "W"],
            scores=[100, 110, 0, 100],
            standing=1,
            division_id=0,
        )
        mockTeam2_2022 = Mock(
            team_id=2,
            owner="Owner 2",
            team_name="Team 2",
            outcomes=["L", "L", "L", "L"],
            scores=[100, 70, 1, 1],
            standing=7,
            division_id=0,
        )
        mockTeam3_2022 = Mock(
            team_id=3,
            owner="Owner 3",
            team_name="Team 3",
            outcomes=["W", "L", "L", "W"],
            scores=[90.5, 100, 80, 2],
            standing=3,
            division_id=0,
        )
        mockTeam4_2022 = Mock(
            team_id=4,
            owner="Owner 4",
            team_name="Team 4",
            outcomes=["L", "W", "U", "L"],
            scores=[70.5, 80, 0, 1],
            standing=5,
            division_id=0,
        )
        mockTeam5_2022 = Mock(
            team_id=5,
            owner="Owner 5",
            team_name="Team 5",
            outcomes=["W", "L", "W", "W"],
            scores=[110, 80, 2, 2],
            standing=4,
            division_id=1,
        )
        mockTeam6_2022 = Mock(
            team_id=6,
            owner="Owner 6",
            team_name="Team 6",
            outcomes=["L", "W", "W", "W"],
            scores=[60, 90, 2, 2],
            standing=6,
            division_id=1,
        )
        mockTeam7_2022 = Mock(
            team_id=7,
            owner="Owner 7",
            team_name="Team 7",
            outcomes=["W", "W", "W", "L"],
            scores=[120, 130, 90, 90],
            standing=2,
            division_id=1,
        )
        mockTeam8_2022 = Mock(
            team_id=8,
            owner="Owner 8",
            team_name="Team 8",
            outcomes=["L", "L", "L", "L"],
            scores=[50, 40, 1, 1],
            standing=8,
            division_id=1,
        )
        # playoff seeds:
        # Team1: 1
        # Team7: 2
        # Team3: 3
        mockTeam1_2022.schedule = [mockTeam2_2022, mockTeam3_2022, mockTeam4_2022, mockTeam7_2022]
        mockTeam2_2022.schedule = [mockTeam1_2022, mockTeam4_2022, mockTeam5_2022, mockTeam3_2022]
        mockTeam3_2022.schedule = [mockTeam4_2022, mockTeam1_2022, mockTeam7_2022, mockTeam2_2022]
        mockTeam4_2022.schedule = [mockTeam3_2022, mockTeam2_2022, mockTeam1_2022, mockTeam5_2022]
        mockTeam5_2022.schedule = [mockTeam6_2022, mockTeam7_2022, mockTeam2_2022, mockTeam4_2022]
        mockTeam6_2022.schedule = [mockTeam5_2022, mockTeam8_2022, mockTeam8_2022, mockTeam8_2022]
        mockTeam7_2022.schedule = [mockTeam8_2022, mockTeam5_2022, mockTeam3_2022, mockTeam1_2022]
        mockTeam8_2022.schedule = [mockTeam7_2022, mockTeam6_2022, mockTeam6_2022, mockTeam6_2022]
        mockEspnLeague2022.teams = [
            mockTeam1_2022,
            mockTeam2_2022,
            mockTeam3_2022,
            mockTeam4_2022,
            mockTeam5_2022,
            mockTeam6_2022,
            mockTeam7_2022,
            mockTeam8_2022,
        ]

        # mock second year (2023)
        mockEspnLeague2023 = Mock()
        mockEspnLeague2023.year = 2023
        mockEspnLeague2023.current_week = 1
        mockEspnLeague2023.settings.name = "Test League 2023"
        mockEspnLeague2023.settings.reg_season_count = 1
        mockEspnLeague2023.settings.division_map = {0: "d1_2023", 1: "d2_2023"}
        mockTeam1_2023 = Mock(
            team_id=1,
            owner="Owner 1",
            team_name="Team 1",
            outcomes=["W"],
            scores=[100],
            division_id=0,
        )
        mockTeam2_2023 = Mock(
            team_id=2,
            owner="Owner 2",
            team_name="Team 2",
            outcomes=["L"],
            scores=[100],
            division_id=0,
        )
        mockTeam3_2023 = Mock(
            team_id=3,
            owner="Owner 3",
            team_name="Team 3",
            outcomes=["W"],
            scores=[90.5],
            division_id=0,
        )
        mockTeam4_2023 = Mock(
            team_id=4,
            owner="Owner 4",
            team_name="Team 4",
            outcomes=["L"],
            scores=[70.5],
            division_id=0,
        )
        mockTeam5_2023 = Mock(
            team_id=5,
            owner="Owner 5",
            team_name="Team 5",
            outcomes=["W"],
            scores=[110],
            division_id=1,
        )
        mockTeam6_2023 = Mock(
            team_id=6,
            owner="Owner 6",
            team_name="Team 6",
            outcomes=["L"],
            scores=[60],
            division_id=1,
        )
        mockTeam7_2023 = Mock(
            team_id=7,
            owner="Owner 7",
            team_name="Team 7",
            outcomes=["W"],
            scores=[120],
            division_id=1,
        )
        mockTeam8_2023 = Mock(
            team_id=8,
            owner="Owner 8",
            team_name="Team 8",
            outcomes=["L"],
            scores=[50],
            division_id=1,
        )
        mockTeam1_2023.schedule = [mockTeam2_2023]
        mockTeam2_2023.schedule = [mockTeam1_2023]
        mockTeam3_2023.schedule = [mockTeam4_2023]
        mockTeam4_2023.schedule = [mockTeam3_2023]
        mockTeam5_2023.schedule = [mockTeam6_2023]
        mockTeam6_2023.schedule = [mockTeam5_2023]
        mockTeam7_2023.schedule = [mockTeam8_2023]
        mockTeam8_2023.schedule = [mockTeam7_2023]
        mockEspnLeague2023.teams = [
            mockTeam1_2023,
            mockTeam2_2023,
            mockTeam3_2023,
            mockTeam4_2023,
            mockTeam5_2023,
            mockTeam6_2023,
            mockTeam7_2023,
            mockTeam8_2023,
        ]

        mockLeague.side_effect = [mockEspnLeague2022, mockEspnLeague2023]

        loader = ESPNLeagueLoader("123", [2022, 2023], leagueName="custom name")
        league = loader.loadLeague()

        self.assertEqual("custom name", league.name)



from leeger.calculator.all_time_calculator import (
    AWALAllTimeCalculator,
    TeamSummaryAllTimeCalculator,
)
from leeger.calculator.all_time_calculator.GameOutcomeAllTimeCalculator import (
    GameOutcomeAllTimeCalculator,
)
from leeger.calculator.all_time_calculator.PlusMinusAllTimeCalculator import (
    PlusMinusAllTimeCalculator,
)
from leeger.calculator.all_time_calculator.PointsScoredAllTimeCalculator import (
    PointsScoredAllTimeCalculator,
)
from leeger.calculator.all_time_calculator.SSLAllTimeCalculator import SSLAllTimeCalculator
from leeger.calculator.all_time_calculator.ScoringShareAllTimeCalculator import (
    ScoringShareAllTimeCalculator,
)
from leeger.calculator.all_time_calculator.ScoringStandardDeviationAllTimeCalculator import (
    ScoringStandardDeviationAllTimeCalculator,
)
from leeger.calculator.all_time_calculator.SingleScoreAllTimeCalculator import (
    SingleScoreAllTimeCalculator,
)
from leeger.calculator.all_time_calculator.SmartWinsAllTimeCalculator import (
    SmartWinsAllTimeCalculator,
)
from leeger.calculator.year_calculator import (
    ScoringStandardDeviationYearCalculator,
    TeamSummaryYearCalculator,
)
from leeger.calculator.year_calculator import SingleScoreYearCalculator
from leeger.calculator.year_calculator.AWALYearCalculator import AWALYearCalculator
from leeger.calculator.year_calculator.GameOutcomeYearCalculator import GameOutcomeYearCalculator
from leeger.calculator.year_calculator.PlusMinusYearCalculator import PlusMinusYearCalculator
from leeger.calculator.year_calculator.PointsScoredYearCalculator import PointsScoredYearCalculator
from leeger.calculator.year_calculator.SSLYearCalculator import SSLYearCalculator
from leeger.calculator.year_calculator.ScoringShareYearCalculator import ScoringShareYearCalculator
from leeger.calculator.year_calculator.SmartWinsYearCalculator import SmartWinsYearCalculator
from leeger.model.league import League, Year
from leeger.model.stat.AllTimeStatSheet import AllTimeStatSheet
from leeger.model.stat.YearStatSheet import YearStatSheet


def leagueStatSheet(league: League, **kwargs) -> AllTimeStatSheet:
    # Team Summary
    gamesPlayed = TeamSummaryAllTimeCalculator.getGamesPlayed(league, **kwargs)

    # Game Outcome
    wins = GameOutcomeAllTimeCalculator.getWins(league, **kwargs)
    losses = GameOutcomeAllTimeCalculator.getLosses(league, **kwargs)
    ties = GameOutcomeAllTimeCalculator.getTies(league, **kwargs)
    winPercentage = GameOutcomeAllTimeCalculator.getWinPercentage(league, **kwargs)
    wal = GameOutcomeAllTimeCalculator.getWAL(league, **kwargs)
    walPerGame = GameOutcomeAllTimeCalculator.getWALPerGame(league, **kwargs)

    # AWAL
    awal = AWALAllTimeCalculator.getAWAL(league, **kwargs)
    awalPerGame = AWALAllTimeCalculator.getAWALPerGame(league, **kwargs)
    opponentAWAL = AWALAllTimeCalculator.getOpponentAWAL(league, **kwargs)
    opponentAWALPerGame = AWALAllTimeCalculator.getOpponentAWALPerGame(league, **kwargs)

    # Smart Wins
    smartWins = SmartWinsAllTimeCalculator.getSmartWins(league, **kwargs)
    smartWinsPerGame = SmartWinsAllTimeCalculator.getSmartWinsPerGame(league, **kwargs)
    opponentSmartWins = SmartWinsAllTimeCalculator.getOpponentSmartWins(league, **kwargs)
    opponentSmartWinsPerGame = SmartWinsAllTimeCalculator.getOpponentSmartWinsPerGame(
        league, **kwargs
    )

    # Points Scored
    pointsScored = PointsScoredAllTimeCalculator.getPointsScored(league, **kwargs)
    pointsScoredPerGame = PointsScoredAllTimeCalculator.getPointsScoredPerGame(league, **kwargs)
    opponentPointsScored = PointsScoredAllTimeCalculator.getOpponentPointsScored(league, **kwargs)
    opponentPointsScoredPerGame = PointsScoredAllTimeCalculator.getOpponentPointsScoredPerGame(
        league, **kwargs
    )

    # Scoring Share
    scoringShare = ScoringShareAllTimeCalculator.getScoringShare(league, **kwargs)
    opponentScoringShare = ScoringShareAllTimeCalculator.getOpponentScoringShare(league, **kwargs)
    maxScoringShare = ScoringShareAllTimeCalculator.getMaxScoringShare(league, **kwargs)
    minScoringShare = ScoringShareAllTimeCalculator.getMinScoringShare(league, **kwargs)

    # Single Score
    maxScore = SingleScoreAllTimeCalculator.getMaxScore(league, **kwargs)
    minScore = SingleScoreAllTimeCalculator.getMinScore(league, **kwargs)

    # Scoring Standard Deviation
    scoringStandardDeviation = (
        ScoringStandardDeviationAllTimeCalculator.getScoringStandardDeviation(league, **kwargs)
    )

    # Plus Minus
    plusMinus = PlusMinusAllTimeCalculator.getPlusMinus(league, **kwargs)

    # SSL
    adjustedTeamScore = SSLAllTimeCalculator.getAdjustedTeamScore(league, **kwargs)
    adjustedTeamSuccess = SSLAllTimeCalculator.getAdjustedTeamSuccess(league, **kwargs)
    adjustedTeamLuck = SSLAllTimeCalculator.getAdjustedTeamLuck(league, **kwargs)

    # check for optional stats
    totalGames = None
    leagueMedianWins = None
    opponentLeagueMedianWins = None
    for year in league.years:
        if year.yearSettings.leagueMedianGames is True:
            totalGames = TeamSummaryAllTimeCalculator.getTotalGames(league, **kwargs)
            leagueMedianWins = GameOutcomeAllTimeCalculator.getLeagueMedianWins(league, **kwargs)
            opponentLeagueMedianWins = GameOutcomeAllTimeCalculator.getOpponentLeagueMedianWins(
                league, **kwargs
            )
            break

    return AllTimeStatSheet(
        gamesPlayed=gamesPlayed,
        wins=wins,
        losses=losses,
        ties=ties,
        winPercentage=winPercentage,
        wal=wal,
        walPerGame=walPerGame,
        awal=awal,
        awalPerGame=awalPerGame,
        opponentAWAL=opponentAWAL,
        opponentAWALPerGame=opponentAWALPerGame,
        smartWins=smartWins,
        smartWinsPerGame=smartWinsPerGame,
        opponentSmartWins=opponentSmartWins,
        opponentSmartWinsPerGame=opponentSmartWinsPerGame,
        pointsScored=pointsScored,
        pointsScoredPerGame=pointsScoredPerGame,
        opponentPointsScored=opponentPointsScored,
        opponentPointsScoredPerGame=opponentPointsScoredPerGame,
        scoringShare=scoringShare,
        opponentScoringShare=opponentScoringShare,
        maxScore=maxScore,
        minScore=minScore,
        scoringStandardDeviation=scoringStandardDeviation,
        plusMinus=plusMinus,
        adjustedTeamScore=adjustedTeamScore,
        adjustedTeamSuccess=adjustedTeamSuccess,
        adjustedTeamLuck=adjustedTeamLuck,
        leagueMedianWins=leagueMedianWins,
        totalGames=totalGames,
        opponentLeagueMedianWins=opponentLeagueMedianWins,
        maxScoringShare=maxScoringShare,
        minScoringShare=minScoringShare,
    )


def yearStatSheet(year: Year, **kwargs) -> YearStatSheet:
    ownerNames = kwargs.pop("ownerNames", None)
    years = kwargs.pop("years", None)
    # Team Summary
    gamesPlayed = TeamSummaryYearCalculator.getGamesPlayed(year, **kwargs)
    # Game Outcome
    wins = GameOutcomeYearCalculator.getWins(year, **kwargs)
    losses = GameOutcomeYearCalculator.getLosses(year, **kwargs)
    ties = GameOutcomeYearCalculator.getTies(year, **kwargs)
    winPercentage = GameOutcomeYearCalculator.getWinPercentage(year, **kwargs)
    wal = GameOutcomeYearCalculator.getWAL(year, **kwargs)
    walPerGame = GameOutcomeYearCalculator.getWALPerGame(year, **kwargs)

    # AWAL
    awal = AWALYearCalculator.getAWAL(year, **kwargs)
    awalPerGame = AWALYearCalculator.getAWALPerGame(year, **kwargs)
    opponentAWAL = AWALYearCalculator.getOpponentAWAL(year, **kwargs)
    opponentAWALPerGame = AWALYearCalculator.getOpponentAWALPerGame(year, **kwargs)

    # Smart Wins
    smartWins = SmartWinsYearCalculator.getSmartWins(year, **kwargs)
    smartWinsPerGame = SmartWinsYearCalculator.getSmartWinsPerGame(year, **kwargs)
    opponentSmartWins = SmartWinsYearCalculator.getOpponentSmartWins(year, **kwargs)
    opponentSmartWinsPerGame = SmartWinsYearCalculator.getOpponentSmartWinsPerGame(year, **kwargs)

    # Points Scored
    pointsScored = PointsScoredYearCalculator.getPointsScored(year, **kwargs)
    pointsScoredPerGame = PointsScoredYearCalculator.getPointsScoredPerGame(year, **kwargs)
    opponentPointsScored = PointsScoredYearCalculator.getOpponentPointsScored(year, **kwargs)
    opponentPointsScoredPerGame = PointsScoredYearCalculator.getOpponentPointsScoredPerGame(
        year, **kwargs
    )

    # Scoring Share
    scoringShare = ScoringShareYearCalculator.getScoringShare(year, **kwargs)
    opponentScoringShare = ScoringShareYearCalculator.getOpponentScoringShare(year, **kwargs)
    maxScoringShare = ScoringShareYearCalculator.getMaxScoringShare(year, **kwargs)
    minScoringShare = ScoringShareYearCalculator.getMinScoringShare(year, **kwargs)

    # Single Score
    maxScore = SingleScoreYearCalculator.getMaxScore(year, **kwargs)
    minScore = SingleScoreYearCalculator.getMinScore(year, **kwargs)

    # Scoring Standard Deviation
    scoringStandardDeviation = ScoringStandardDeviationYearCalculator.getScoringStandardDeviation(
        year, **kwargs
    )

    # Plus Minus
    plusMinus = PlusMinusYearCalculator.getPlusMinus(year, **kwargs)

    # SSL
    teamScore = SSLYearCalculator.getTeamScore(year, **kwargs)
    teamSuccess = SSLYearCalculator.getTeamSuccess(year, **kwargs)
    teamLuck = SSLYearCalculator.getTeamLuck(year, **kwargs)

    # check for optional stats
    totalGames = None
    leagueMedianWins = None
    opponentLeagueMedianWins = None
    if year.yearSettings.leagueMedianGames is True:
        totalGames = TeamSummaryYearCalculator.getTotalGames(year, **kwargs)
        leagueMedianWins = GameOutcomeYearCalculator.getLeagueMedianWins(year, **kwargs)
        opponentLeagueMedianWins = GameOutcomeYearCalculator.getOpponentLeagueMedianWins(
            year, **kwargs
        )

    return YearStatSheet(
        gamesPlayed=gamesPlayed,
        wins=wins,
        losses=losses,
        ties=ties,
        winPercentage=winPercentage,
        wal=wal,
        walPerGame=walPerGame,
        awal=awal,
        awalPerGame=awalPerGame,
        opponentAWAL=opponentAWAL,
        opponentAWALPerGame=opponentAWALPerGame,
        smartWins=smartWins,
        smartWinsPerGame=smartWinsPerGame,
        opponentSmartWins=opponentSmartWins,
        opponentSmartWinsPerGame=opponentSmartWinsPerGame,
        pointsScored=pointsScored,
        pointsScoredPerGame=pointsScoredPerGame,
        opponentPointsScored=opponentPointsScored,
        opponentPointsScoredPerGame=opponentPointsScoredPerGame,
        scoringShare=scoringShare,
        opponentScoringShare=opponentScoringShare,
        maxScore=maxScore,
        minScore=minScore,
        scoringStandardDeviation=scoringStandardDeviation,
        plusMinus=plusMinus,
        teamScore=teamScore,
        teamSuccess=teamSuccess,
        teamLuck=teamLuck,
        leagueMedianWins=leagueMedianWins,
        totalGames=totalGames,
        opponentLeagueMedianWins=opponentLeagueMedianWins,
        maxScoringShare=maxScoringShare,
        minScoringShare=minScoringShare,
        ownerNames=ownerNames,
        years=years,
    )



from .Division import Division
from .League import League
from .Matchup import Matchup
from .Owner import Owner
from .Team import Team
from .Week import Week
from .Year import Year
from .YearSettings import YearSettings



import unittest

from leeger.util.GeneralUtil import GeneralUtil


class TestGeneralUtil(unittest.TestCase):
    def test_filter_None(self):
        l = [None, "test", None]
        response = GeneralUtil.filter(value=None, list_=l)
        self.assertEqual(["test"], response)

    def test_filter_emptyList(self):
        l = list()
        response = GeneralUtil.filter(value=None, list_=l)
        self.assertEqual([], response)

    def test_warnForUnusedKwargs_happyPath(self):
        kwargs = {"a": 1, "b": 2}
        with self.assertLogs() as captured:
            GeneralUtil.warnForUnusedKwargs(kwargs)
        self.assertEqual(2, len(captured.records))
        self.assertEqual("Keyword argument 'a' unused.", captured.records[0].getMessage())
        self.assertEqual("Keyword argument 'b' unused.", captured.records[1].getMessage())

        # with default exclude keys
        kwargs = {"b": 1, "validate": 2}
        with self.assertLogs() as captured:
            GeneralUtil.warnForUnusedKwargs(kwargs)
        self.assertEqual(1, len(captured.records))
        self.assertEqual("Keyword argument 'b' unused.", captured.records[0].getMessage())

        # with exclude keys
        kwargs = {"a": 1, "b": 2}
        with self.assertLogs() as captured:
            GeneralUtil.warnForUnusedKwargs(kwargs, excludeKeys=["a"])
        self.assertEqual(1, len(captured.records))
        self.assertEqual("Keyword argument 'b' unused.", captured.records[0].getMessage())

    def test_safeSum_happyPath(self):
        response = GeneralUtil.safeSum(None, 1, 1)
        self.assertEqual(2, response)

        response = GeneralUtil.safeSum(1, None, 1)
        self.assertEqual(2, response)

        response = GeneralUtil.safeSum(1, 1, None)
        self.assertEqual(2, response)

        response = GeneralUtil.safeSum(None, None, None)
        self.assertIsNone(response)

        response = GeneralUtil.safeSum(1, 1, 1)
        self.assertEqual(3, response)

    def test_findDifferentFields_noDifference_simpleDict(self):
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "baz", "bar": "bot"}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual(list(), response)

        # with list
        d1 = {"foo": ["baz", "boy"], "bar": "bot"}
        d2 = {"foo": ["baz", "boy"], "bar": "bot"}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual(list(), response)

        # with tuple
        d1 = {"foo": ("baz", "boy"), "bar": "bot"}
        d2 = {"foo": ("baz", "boy"), "bar": "bot"}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual(list(), response)

        # with ignore key names (some keys)
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "baz", "bar": "bot"}
        response = GeneralUtil.findDifferentFields(d1, d2, ignoreKeyNames=["foo"])
        self.assertEqual(list(), response)

        # with ignore key names (all keys)
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "baz", "bar": "bot"}
        response = GeneralUtil.findDifferentFields(d1, d2, ignoreKeyNames=["foo", "bar"])
        self.assertEqual(list(), response)

        # with parent key given
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "baz", "bar": "bot"}
        response = GeneralUtil.findDifferentFields(d1, d2, parentKey="abc")
        self.assertEqual(list(), response)

    def test_findDifferentFields_someDifference_simpleDict(self):
        # single field difference
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "ba", "bar": "bot"}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo", ("baz", "ba"))], response)

        # list differences
        # single field difference with same list length
        d1 = {"foo": ["baz", "bar"]}
        d2 = {"foo": ["baz", "ba"]}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo[1]", ("bar", "ba"))], response)

        # all values in list differ, shows key as difference
        d1 = {"foo": ["baz", "bar", "bot", "boy"]}
        d2 = {"foo": ["ba", "ba", "bo", "bo"]}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual(
            [("foo", (["baz", "bar", "bot", "boy"], ["ba", "ba", "bo", "bo"]))], response
        )

        # lists have unequal length, shows key as difference
        d1 = {"foo": ["baz", "bar"]}
        d2 = {"foo": ["baz"]}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo", (["baz", "bar"], ["baz"]))], response)

        # tuple differences
        # single field difference with same list length
        d1 = {"foo": ("baz", "bar")}
        d2 = {"foo": ("baz", "ba")}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo[1]", ("bar", "ba"))], response)

        # all values in tuples differ, shows key as difference
        d1 = {"foo": ("baz", "bar", "bot", "boy")}
        d2 = {"foo": ("ba", "ba", "bo", "bo")}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual(
            [("foo", (("baz", "bar", "bot", "boy"), ("ba", "ba", "bo", "bo")))], response
        )

        # tuples have unequal length, shows key as difference
        d1 = {"foo": ("baz", "bar")}
        d2 = {"foo": ("baz")}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo", (("baz", "bar"), ("baz")))], response)

        # multiple field differences
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "ba", "bar": "bo"}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo", ("baz", "ba")), ("bar", ("bot", "bo"))], response)

        # difference with ignore key names (some keys)
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "ba", "bar": "bo"}
        response = GeneralUtil.findDifferentFields(d1, d2, ignoreKeyNames=["foo"])
        self.assertEqual([("bar", ("bot", "bo"))], response)

        # difference with ignore key names (all difference keys)
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "ba", "bar": "bo"}
        response = GeneralUtil.findDifferentFields(d1, d2, ignoreKeyNames=["foo", "bar"])
        self.assertEqual(list(), response)

        # with parent key given
        d1 = {"foo": "baz", "bar": "bot"}
        d2 = {"foo": "ba", "bar": "bot"}
        response = GeneralUtil.findDifferentFields(d1, d2, parentKey="abc")
        self.assertEqual([("abc.foo", ("baz", "ba"))], response)

    def test_findDifferentFields_nestedDict(self):
        # with dict nested
        d1 = {"foo": {"baz": "bar"}, "bot": {"boo": "boy"}}
        d2 = {"foo": {"baz": "ba"}, "bot": {"boo": "bo"}}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo.baz", ("bar", "ba")), ("bot.boo", ("boy", "bo"))], response)

        # with list nested
        # same list length, single index difference
        d1 = {"foo": {"bar": ["bot", "boy"]}}
        d2 = {"foo": {"bar": ["bot", "bo"]}}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo.bar[1]", ("boy", "bo"))], response)

        # same list length, all index difference
        d1 = {"foo": {"bar": ["bot", "boy"]}}
        d2 = {"foo": {"bar": ["bo", "bo"]}}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo.bar", (["bot", "boy"], ["bo", "bo"]))], response)

        # same list length, many (not all) index difference
        d1 = {"foo": {"bar": ["bot", "boy", "boz"]}}
        d2 = {"foo": {"bar": ["bo", "bo", "boz"]}}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo.bar[0]", ("bot", "bo")), ("foo.bar[1]", ("boy", "bo"))], response)

        # different list length
        d1 = {"foo": {"bar": ["bot", "boy"]}}
        d2 = {"foo": {"bar": ["bot"]}}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo.bar", (["bot", "boy"], ["bot"]))], response)

        # with ignore key names (some keys)
        d1 = {"foo": {"baz": "bar"}}
        d2 = {"foo": {"baz": "ba"}}
        response = GeneralUtil.findDifferentFields(d1, d2, ignoreKeyNames=["baz"])
        self.assertEqual(list(), response)

        # list with ignore key names
        d1 = {"foo": {"bar": [{"bot": "boy"}]}}
        d2 = {"foo": {"bar": [{"bot": "bo"}]}}
        response = GeneralUtil.findDifferentFields(d1, d2)
        self.assertEqual([("foo.bar[0].bot", ("boy", "bo"))], response)

        # with parent key given
        d1 = {"foo": {"baz": "bar"}, "bot": {"boo": "boy"}}
        d2 = {"foo": {"baz": "ba"}, "bot": {"boo": "bo"}}
        response = GeneralUtil.findDifferentFields(d1, d2, parentKey="abc")
        self.assertEqual([("abc.foo.baz", ("bar", "ba")), ("abc.bot.boo", ("boy", "bo"))], response)



import unittest
from leeger.exception.InvalidDivisionFormatException import InvalidDivisionFormatException

from leeger.model.league.Division import Division
from leeger.validate import divisionValidation


class TestDivisionValidation(unittest.TestCase):
    def test_checkAllTypes_divisionNameIsntTypeStr_raisesException(self):
        with self.assertRaises(InvalidDivisionFormatException) as context:
            divisionValidation.checkAllTypes(Division(name=None))
        self.assertEqual("name must be type 'str'.", str(context.exception))



from __future__ import annotations

from dataclasses import dataclass
from typing import Optional
from leeger.model.abstract.EqualityCheck import EqualityCheck
from leeger.util.CustomLogger import CustomLogger

from leeger.util.JSONDeserializable import JSONDeserializable
from leeger.util.JSONSerializable import JSONSerializable
from leeger.util.equality import modelEquals


@dataclass(kw_only=True, eq=False)
class YearSettings(EqualityCheck, JSONSerializable, JSONDeserializable):
    __LOGGER = CustomLogger.getLogger()
    leagueMedianGames: Optional[bool] = False

    def __post_init__(self):
        if self.leagueMedianGames is None:
            self.leagueMedianGames = False

    def equals(
        self,
        otherYearSettings: YearSettings,
        *,
        ignoreIds: bool = False,
        ignoreBaseIds: bool = False,
        logDifferences: bool = False,
    ) -> bool:
        """
        Checks if *this* YearSettings is the same as the given YearSettings.
        """

        return modelEquals(
            objA=self,
            objB=otherYearSettings,
            baseFields={"leagueMedianGames"},
            parentKey="YearSettings",
            ignoreIdFields=ignoreIds,
            ignoreBaseIdField=ignoreBaseIds,
            logDifferences=logDifferences,
        )

    def __eq__(self, otherYearSettings: YearSettings) -> bool:
        self.__LOGGER.info("Use .equals() for more options when comparing YearSettings instances.")
        return self.equals(otherYearSettings=otherYearSettings)

    def toJson(self) -> dict:
        return {"leagueMedianGames": self.leagueMedianGames}

    @staticmethod
    def fromJson(d: dict) -> YearSettings:
        return YearSettings(leagueMedianGames=d.get("leagueMedianGames"))



import unittest
from unittest import mock
from unittest.mock import Mock
import copy
from leeger.enum.MatchupType import MatchupType

from leeger.league_loader import YahooLeagueLoader
from leeger.model.league.League import League
from leeger.model.league.Matchup import Matchup
from leeger.model.league.Owner import Owner
from leeger.model.league.Team import Team
from leeger.model.league.Week import Week
from leeger.model.league.Year import Year


class TestYahooLeagueLoader(unittest.TestCase):
    # helper methods
    def __getMockTeamsMethod(self, teams: list) -> callable:
        def mockTeamsMethod():
            return teams

        return mockTeamsMethod

    def __getMockWeeksMethod(self, weeks: list) -> callable:
        def mockWeeksMethod():
            return weeks

        return mockWeeksMethod

    def __getMockYahooTeam(
        self, *, teamId: int, teamKey: int, name: str, managerNickname: str, managerId: int
    ) -> Mock:
        mockTeam = Mock()
        mockTeam.team_id = teamId
        mockTeam.team_key = teamKey
        mockTeam.name = name
        mockTeam.manager.nickname = managerNickname
        mockTeam.manager.manager_id = managerId
        return mockTeam

    def __getMockYahooMatchup(
        self,
        *,
        week: int,
        status: str,
        winnerTeamKey: int,
        isTied: int,
        isPlayoffs: int,
        isConsolation: int,
        league: Mock
    ) -> Mock:
        # duplicate mockYahooTeam to avoid using the same object in memory
        dupLeague = copy.deepcopy(league)
        mockYahooMatchup = Mock()
        mockYahooMatchup.week = week
        mockYahooMatchup.status = status
        mockYahooMatchup.winner_team_key = winnerTeamKey
        mockYahooMatchup.is_tied = isTied
        mockYahooMatchup.is_playoffs = isPlayoffs
        mockYahooMatchup.is_consolation = isConsolation
        mockYahooMatchup.league = dupLeague
        return mockYahooMatchup

    def __setMockYahooTeamPoints(self, mockYahooTeam: Mock, teamPointsTotal: int | float) -> Mock:
        # duplicate mockYahooTeam to avoid using the same object in memory
        dupMockYahooTeam = copy.deepcopy(mockYahooTeam)
        dupMockYahooTeam.team_points.total = teamPointsTotal
        return dupMockYahooTeam

    def test_loadLeague_nonIntPassingStringForLeagueId(self):
        with self.assertRaises(ValueError) as context:
            badClientId = badClientSecret = "bad"
            badLeagueId = "a"
            YahooLeagueLoader(
                badLeagueId,
                [2000],
                clientId=badClientId,
                clientSecret=badClientSecret,
                loginTimeoutSeconds=1,
            )
        self.assertEqual("League ID 'a' could not be turned into an int.", str(context.exception))

    @mock.patch("multiprocessing.Process")
    def test_loadLeague_intendedFailure(self, mockMultiprocessingProcess):
        mockLoginProcess = mockMultiprocessingProcess.return_value
        mockLoginProcess.is_alive.return_value = True  # Simulate login process failure
        with self.assertRaises(TimeoutError) as context:
            yahooLeagueLoader = YahooLeagueLoader("123", [2022], clientId="cid", clientSecret="cs")
            yahooLeagueLoader.loadLeague()
        self.assertEqual("Login to yahoofantasy timed out.", str(context.exception))

    @mock.patch("multiprocessing.Process")
    @mock.patch("yahoofantasy.Context.__init__")
    @mock.patch("yahoofantasy.Context.get_leagues")
    def test_loadLeague_happyPath(
        self, mockYahooContextGetLeagues, mockYahooContextInit, mockMultiprocessingProcess
    ):
        # mock real league 2022
        mockLeague2022 = Mock()
        mockLeague2022.name = "Test League 2022"
        mockLeague2022.league_id = "123"
        mockLeague2022.season = 2022
        mockLeague2022.current_week = 3
        mockLeague2022.end_week = 3

        # mock fake leagues 2022
        mockLeague2022_fake1 = Mock()
        mockLeague2022_fake1.league_id = "456"

        mockLeague2022_fake2 = Mock()
        mockLeague2022_fake2.league_id = "999"

        # mock league 2023
        mockLeague2023 = Mock()
        mockLeague2023.name = "Test League 2023"
        mockLeague2023.league_id = "456"
        mockLeague2023.season = 2023
        mockLeague2023.current_week = 3
        mockLeague2023.end_week = 3
        mockLeague2023.past_league_id = ["", "123"]

        # mock fake leagues 2023
        mockLeague2023_fake1 = Mock()
        mockLeague2023_fake1.league_id = "123"

        mockLeague2023_fake2 = Mock()
        mockLeague2023_fake2.league_id = "999"

        # mock teams 2022
        mockYahooTeam1_2022 = self.__getMockYahooTeam(
            teamId=1, teamKey=1, name="Team 1", managerNickname="Owner 1", managerId=1
        )
        mockYahooTeam2_2022 = self.__getMockYahooTeam(
            teamId=2, teamKey=2, name="Team 2", managerNickname="Owner 2", managerId=2
        )
        mockYahooTeam3_2022 = self.__getMockYahooTeam(
            teamId=3, teamKey=3, name="Team 3", managerNickname="Owner 3", managerId=3
        )
        mockYahooTeam4_2022 = self.__getMockYahooTeam(
            teamId=4, teamKey=4, name="Team 4", managerNickname="Owner 4", managerId=4
        )
        mockYahooTeam5_2022 = self.__getMockYahooTeam(
            teamId=5, teamKey=5, name="Team 5", managerNickname="Owner 5", managerId=5
        )
        mockYahooTeam6_2022 = self.__getMockYahooTeam(
            teamId=6, teamKey=6, name="Team 6", managerNickname="Owner 6", managerId=6
        )
        mockYahooTeam7_2022 = self.__getMockYahooTeam(
            teamId=7, teamKey=7, name="Team 7", managerNickname="Owner 7", managerId=7
        )
        mockYahooTeam8_2022 = self.__getMockYahooTeam(
            teamId=8, teamKey=8, name="Team 8", managerNickname="Owner 8", managerId=8
        )
        mockLeague2022.teams = self.__getMockTeamsMethod(
            [
                mockYahooTeam1_2022,
                mockYahooTeam2_2022,
                mockYahooTeam3_2022,
                mockYahooTeam4_2022,
                mockYahooTeam5_2022,
                mockYahooTeam6_2022,
                mockYahooTeam7_2022,
                mockYahooTeam8_2022,
            ]
        )

        # mock teams 2023
        mockYahooTeam1_2023 = self.__getMockYahooTeam(
            teamId=1, teamKey=1, name="Team 1", managerNickname="Owner 1", managerId=1
        )
        mockYahooTeam2_2023 = self.__getMockYahooTeam(
            teamId=2, teamKey=2, name="Team 2", managerNickname="Owner 2", managerId=2
        )
        mockYahooTeam3_2023 = self.__getMockYahooTeam(
            teamId=3, teamKey=3, name="Team 3", managerNickname="Owner 3", managerId=3
        )
        mockYahooTeam4_2023 = self.__getMockYahooTeam(
            teamId=4, teamKey=4, name="Team 4", managerNickname="Owner 4", managerId=4
        )
        mockYahooTeam5_2023 = self.__getMockYahooTeam(
            teamId=5, teamKey=5, name="Team 5", managerNickname="Owner 5", managerId=5
        )
        mockYahooTeam6_2023 = self.__getMockYahooTeam(
            teamId=6, teamKey=6, name="Team 6", managerNickname="Owner 6", managerId=6
        )
        mockYahooTeam7_2023 = self.__getMockYahooTeam(
            teamId=7, teamKey=7, name="Team 7", managerNickname="Owner 7", managerId=7
        )
        mockYahooTeam8_2023 = self.__getMockYahooTeam(
            teamId=8, teamKey=8, name="Team 8", managerNickname="Owner 8", managerId=8
        )
        mockLeague2023.teams = self.__getMockTeamsMethod(
            [
                mockYahooTeam1_2023,
                mockYahooTeam2_2023,
                mockYahooTeam3_2023,
                mockYahooTeam4_2023,
                mockYahooTeam5_2023,
                mockYahooTeam6_2023,
                mockYahooTeam7_2023,
                mockYahooTeam8_2023,
            ]
        )

        # mock matchups 2022
        # week -> matchup number -> year
        mockYahooMatchup1_1_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=100),
        ]
        mockYahooMatchup1_2_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=3,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_2_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2022, teamPointsTotal=100.1),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam4_2022, teamPointsTotal=90.1),
        ]
        mockYahooMatchup1_3_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=5,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_3_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam5_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam6_2022, teamPointsTotal=90),
        ]
        mockYahooMatchup1_4_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=7,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_4_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam7_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam8_2022, teamPointsTotal=90),
        ]
        # playoffs
        mockYahooMatchup2_1_2022 = self.__getMockYahooMatchup(
            week=2,
            status="postevent",
            winnerTeamKey=2,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup2_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=101),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2022, teamPointsTotal=91),
        ]
        # championship
        mockYahooMatchup3_1_2022 = self.__getMockYahooMatchup(
            week=3,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup3_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2022, teamPointsTotal=102),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=92),
        ]

        # mock matchups 2023
        # week -> matchup number -> year
        mockYahooMatchup1_1_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=100),
        ]
        mockYahooMatchup1_2_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=3,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_2_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2023, teamPointsTotal=100.1),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam4_2023, teamPointsTotal=90.1),
        ]
        mockYahooMatchup1_3_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=5,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_3_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam5_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam6_2023, teamPointsTotal=90),
        ]
        mockYahooMatchup1_4_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=7,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_4_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam7_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam8_2023, teamPointsTotal=90),
        ]
        # playoffs
        mockYahooMatchup2_1_2023 = self.__getMockYahooMatchup(
            week=2,
            status="postevent",
            winnerTeamKey=2,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup2_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=101),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2023, teamPointsTotal=91),
        ]
        # championship
        mockYahooMatchup3_1_2023 = self.__getMockYahooMatchup(
            week=3,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup3_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2023, teamPointsTotal=102),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=92),
        ]

        # mock weeks 2022
        mockWeek1_2022 = Mock()
        mockWeek1_2022.matchups = [
            mockYahooMatchup1_1_2022,
            mockYahooMatchup1_2_2022,
            mockYahooMatchup1_3_2022,
            mockYahooMatchup1_4_2022,
        ]

        mockWeek2_2022 = Mock()
        mockWeek2_2022.matchups = [mockYahooMatchup2_1_2022]

        mockWeek3_2022 = Mock()
        mockWeek3_2022.matchups = [mockYahooMatchup3_1_2022]

        mockLeague2022.weeks = self.__getMockWeeksMethod(
            [mockWeek1_2022, mockWeek2_2022, mockWeek3_2022]
        )

        # mock weeks 2023
        mockWeek1_2023 = Mock()
        mockWeek1_2023.matchups = [
            mockYahooMatchup1_1_2023,
            mockYahooMatchup1_2_2023,
            mockYahooMatchup1_3_2023,
            mockYahooMatchup1_4_2023,
        ]

        mockWeek2_2023 = Mock()
        mockWeek2_2023.matchups = [mockYahooMatchup2_1_2023]

        mockWeek3_2023 = Mock()
        mockWeek3_2023.matchups = [mockYahooMatchup3_1_2023]

        mockLeague2023.weeks = self.__getMockWeeksMethod(
            [mockWeek1_2023, mockWeek2_2023, mockWeek3_2023]
        )

        mockYahooContextInit.side_effect = [None]
        mockYahooContextGetLeagues.side_effect = [
            [mockLeague2023_fake1, mockLeague2023, mockLeague2023_fake2],
            [mockLeague2022_fake1, mockLeague2022, mockLeague2022_fake2],
        ]
        mockLoginProcess = mockMultiprocessingProcess.return_value
        mockLoginProcess.is_alive.return_value = False  # Simulate login process completion

        # load league
        yahooLeagueLoader = YahooLeagueLoader(
            "456", [2022, 2023], clientId="cid", clientSecret="cs"
        )
        league = yahooLeagueLoader.loadLeague()

        # expected league
        team1_2022 = Team(ownerId=1, name="Team 1")
        team2_2022 = Team(ownerId=2, name="Team 2")
        team3_2022 = Team(ownerId=3, name="Team 3")
        team4_2022 = Team(ownerId=4, name="Team 4")
        team5_2022 = Team(ownerId=5, name="Team 5")
        team6_2022 = Team(ownerId=6, name="Team 6")
        team7_2022 = Team(ownerId=7, name="Team 7")
        team8_2022 = Team(ownerId=8, name="Team 8")

        team1_2023 = Team(ownerId=1, name="Team 1")
        team2_2023 = Team(ownerId=2, name="Team 2")
        team3_2023 = Team(ownerId=3, name="Team 3")
        team4_2023 = Team(ownerId=4, name="Team 4")
        team5_2023 = Team(ownerId=5, name="Team 5")
        team6_2023 = Team(ownerId=6, name="Team 6")
        team7_2023 = Team(ownerId=7, name="Team 7")
        team8_2023 = Team(ownerId=8, name="Team 8")

        expectedLeague = League(
            name="Test League 2023",
            owners=[
                Owner(name="Owner 1"),
                Owner(name="Owner 2"),
                Owner(name="Owner 3"),
                Owner(name="Owner 4"),
                Owner(name="Owner 5"),
                Owner(name="Owner 6"),
                Owner(name="Owner 7"),
                Owner(name="Owner 8"),
            ],
            years=[
                Year(
                    yearNumber=2022,
                    teams=[
                        team1_2022,
                        team2_2022,
                        team3_2022,
                        team4_2022,
                        team5_2022,
                        team6_2022,
                        team7_2022,
                        team8_2022,
                    ],
                    weeks=[
                        Week(
                            weekNumber=1,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team2_2022.id,
                                    teamAScore=100,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2022.id,
                                    teamBId=team4_2022.id,
                                    teamAScore=100.1,
                                    teamBScore=90.1,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2022.id,
                                    teamBId=team6_2022.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team7_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=2,
                            matchups=[
                                Matchup(
                                    teamAId=team2_2022.id,
                                    teamBId=team3_2022.id,
                                    teamAScore=101,
                                    teamBScore=91,
                                    matchupType=MatchupType.PLAYOFF,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                )
                            ],
                        ),
                        Week(
                            weekNumber=3,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team2_2022.id,
                                    teamAScore=102,
                                    teamBScore=92,
                                    matchupType=MatchupType.CHAMPIONSHIP,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                )
                            ],
                        ),
                    ],
                    yearSettings=None,
                ),
                Year(
                    yearNumber=2023,
                    teams=[
                        team1_2023,
                        team2_2023,
                        team3_2023,
                        team4_2023,
                        team5_2023,
                        team6_2023,
                        team7_2023,
                        team8_2023,
                    ],
                    weeks=[
                        Week(
                            weekNumber=1,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2023.id,
                                    teamBId=team2_2023.id,
                                    teamAScore=100,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2023.id,
                                    teamBId=team4_2023.id,
                                    teamAScore=100.1,
                                    teamBScore=90.1,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2023.id,
                                    teamBId=team6_2023.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team7_2023.id,
                                    teamBId=team8_2023.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=2,
                            matchups=[
                                Matchup(
                                    teamAId=team2_2023.id,
                                    teamBId=team3_2023.id,
                                    teamAScore=101,
                                    teamBScore=91,
                                    matchupType=MatchupType.PLAYOFF,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                )
                            ],
                        ),
                        Week(
                            weekNumber=3,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2023.id,
                                    teamBId=team2_2023.id,
                                    teamAScore=102,
                                    teamBScore=92,
                                    matchupType=MatchupType.CHAMPIONSHIP,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                )
                            ],
                        ),
                    ],
                    yearSettings=None,
                ),
            ],
        )

        # make sure we called login correctly
        mockMultiprocessingProcess.assert_called_once_with(
            target=yahooLeagueLoader.login, args=("cid", "cs")
        )

        self.assertTrue(league.equals(expectedLeague, ignoreBaseIds=True, ignoreIds=True))
        # check multiWeekMatchupIds
        for year in league.years:
            for week in year.weeks:
                for matchup in week.matchups:
                    self.assertIsNone(matchup.multiWeekMatchupId)

    @mock.patch("multiprocessing.Process")
    @mock.patch("yahoofantasy.Context.__init__")
    @mock.patch("yahoofantasy.Context.get_leagues")
    def test_loadLeague_withOwnerNamesAndAliases(
        self, mockYahooContextGetLeagues, mockYahooContextInit, mockMultiprocessingProcess
    ):
        # mock real league 2022
        mockLeague2022 = Mock()
        mockLeague2022.name = "Test League 2022"
        mockLeague2022.league_id = "123"
        mockLeague2022.season = 2022
        mockLeague2022.current_week = 3
        mockLeague2022.end_week = 3

        # mock fake leagues 2022
        mockLeague2022_fake1 = Mock()
        mockLeague2022_fake1.league_id = "456"

        mockLeague2022_fake2 = Mock()
        mockLeague2022_fake2.league_id = "999"

        # mock league 2023
        mockLeague2023 = Mock()
        mockLeague2023.name = "Test League 2023"
        mockLeague2023.league_id = "456"
        mockLeague2023.season = 2023
        mockLeague2023.current_week = 3
        mockLeague2023.end_week = 3
        mockLeague2023.past_league_id = ["", "123"]

        # mock fake leagues 2023
        mockLeague2023_fake1 = Mock()
        mockLeague2023_fake1.league_id = "123"

        mockLeague2023_fake2 = Mock()
        mockLeague2023_fake2.league_id = "999"

        # mock teams 2022
        mockYahooTeam1_2022 = self.__getMockYahooTeam(
            teamId=1, teamKey=1, name="Team 1", managerNickname="Owner 1", managerId=1
        )
        mockYahooTeam2_2022 = self.__getMockYahooTeam(
            teamId=2, teamKey=2, name="Team 2", managerNickname="Owner 2", managerId=2
        )
        mockYahooTeam3_2022 = self.__getMockYahooTeam(
            teamId=3, teamKey=3, name="Team 3", managerNickname="Owner 3", managerId=3
        )
        mockYahooTeam4_2022 = self.__getMockYahooTeam(
            teamId=4, teamKey=4, name="Team 4", managerNickname="Owner 4", managerId=4
        )
        mockYahooTeam5_2022 = self.__getMockYahooTeam(
            teamId=5, teamKey=5, name="Team 5", managerNickname="Owner 5", managerId=5
        )
        mockYahooTeam6_2022 = self.__getMockYahooTeam(
            teamId=6, teamKey=6, name="Team 6", managerNickname="Owner 6", managerId=6
        )
        mockYahooTeam7_2022 = self.__getMockYahooTeam(
            teamId=7, teamKey=7, name="Team 7", managerNickname="Owner 7", managerId=7
        )
        mockYahooTeam8_2022 = self.__getMockYahooTeam(
            teamId=8, teamKey=8, name="Team 8", managerNickname="Owner 8", managerId=8
        )
        mockLeague2022.teams = self.__getMockTeamsMethod(
            [
                mockYahooTeam1_2022,
                mockYahooTeam2_2022,
                mockYahooTeam3_2022,
                mockYahooTeam4_2022,
                mockYahooTeam5_2022,
                mockYahooTeam6_2022,
                mockYahooTeam7_2022,
                mockYahooTeam8_2022,
            ]
        )

        # mock teams 2023
        mockYahooTeam1_2023 = self.__getMockYahooTeam(
            teamId=1, teamKey=1, name="Team 1", managerNickname="Owner 1", managerId=1
        )
        mockYahooTeam2_2023 = self.__getMockYahooTeam(
            teamId=2, teamKey=2, name="Team 2", managerNickname="Owner 2", managerId=2
        )
        mockYahooTeam3_2023 = self.__getMockYahooTeam(
            teamId=3, teamKey=3, name="Team 3", managerNickname="Owner 3", managerId=3
        )
        mockYahooTeam4_2023 = self.__getMockYahooTeam(
            teamId=4, teamKey=4, name="Team 4", managerNickname="Owner 4", managerId=4
        )
        mockYahooTeam5_2023 = self.__getMockYahooTeam(
            teamId=5, teamKey=5, name="Team 5", managerNickname="Owner 5", managerId=5
        )
        mockYahooTeam6_2023 = self.__getMockYahooTeam(
            teamId=6, teamKey=6, name="Team 6", managerNickname="Owner 6", managerId=6
        )
        mockYahooTeam7_2023 = self.__getMockYahooTeam(
            teamId=7, teamKey=7, name="Team 7", managerNickname="Owner 7", managerId=7
        )
        mockYahooTeam8_2023 = self.__getMockYahooTeam(
            teamId=8, teamKey=8, name="Team 8", managerNickname="Owner 8", managerId=8
        )
        mockLeague2023.teams = self.__getMockTeamsMethod(
            [
                mockYahooTeam1_2023,
                mockYahooTeam2_2023,
                mockYahooTeam3_2023,
                mockYahooTeam4_2023,
                mockYahooTeam5_2023,
                mockYahooTeam6_2023,
                mockYahooTeam7_2023,
                mockYahooTeam8_2023,
            ]
        )

        # mock matchups 2022
        # week -> matchup number -> year
        mockYahooMatchup1_1_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=100),
        ]
        mockYahooMatchup1_2_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=3,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_2_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2022, teamPointsTotal=100.1),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam4_2022, teamPointsTotal=90.1),
        ]
        mockYahooMatchup1_3_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=5,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_3_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam5_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam6_2022, teamPointsTotal=90),
        ]
        mockYahooMatchup1_4_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=7,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_4_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam7_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam8_2022, teamPointsTotal=90),
        ]
        # playoffs
        mockYahooMatchup2_1_2022 = self.__getMockYahooMatchup(
            week=2,
            status="postevent",
            winnerTeamKey=2,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup2_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=101),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2022, teamPointsTotal=91),
        ]
        # championship
        mockYahooMatchup3_1_2022 = self.__getMockYahooMatchup(
            week=3,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup3_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2022, teamPointsTotal=102),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=92),
        ]

        # mock matchups 2023
        # week -> matchup number -> year
        mockYahooMatchup1_1_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=100),
        ]
        mockYahooMatchup1_2_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=3,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_2_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2023, teamPointsTotal=100.1),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam4_2023, teamPointsTotal=90.1),
        ]
        mockYahooMatchup1_3_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=5,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_3_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam5_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam6_2023, teamPointsTotal=90),
        ]
        mockYahooMatchup1_4_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=7,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_4_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam7_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam8_2023, teamPointsTotal=90),
        ]
        # playoffs
        mockYahooMatchup2_1_2023 = self.__getMockYahooMatchup(
            week=2,
            status="postevent",
            winnerTeamKey=2,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup2_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=101),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2023, teamPointsTotal=91),
        ]
        # championship
        mockYahooMatchup3_1_2023 = self.__getMockYahooMatchup(
            week=3,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup3_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2023, teamPointsTotal=102),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=92),
        ]

        # mock weeks 2022
        mockWeek1_2022 = Mock()
        mockWeek1_2022.matchups = [
            mockYahooMatchup1_1_2022,
            mockYahooMatchup1_2_2022,
            mockYahooMatchup1_3_2022,
            mockYahooMatchup1_4_2022,
        ]

        mockWeek2_2022 = Mock()
        mockWeek2_2022.matchups = [mockYahooMatchup2_1_2022]

        mockWeek3_2022 = Mock()
        mockWeek3_2022.matchups = [mockYahooMatchup3_1_2022]

        mockLeague2022.weeks = self.__getMockWeeksMethod(
            [mockWeek1_2022, mockWeek2_2022, mockWeek3_2022]
        )

        # mock weeks 2023
        mockWeek1_2023 = Mock()
        mockWeek1_2023.matchups = [
            mockYahooMatchup1_1_2023,
            mockYahooMatchup1_2_2023,
            mockYahooMatchup1_3_2023,
            mockYahooMatchup1_4_2023,
        ]

        mockWeek2_2023 = Mock()
        mockWeek2_2023.matchups = [mockYahooMatchup2_1_2023]

        mockWeek3_2023 = Mock()
        mockWeek3_2023.matchups = [mockYahooMatchup3_1_2023]

        mockLeague2023.weeks = self.__getMockWeeksMethod(
            [mockWeek1_2023, mockWeek2_2023, mockWeek3_2023]
        )

        mockYahooContextInit.side_effect = [None]
        mockYahooContextGetLeagues.side_effect = [
            [mockLeague2023_fake1, mockLeague2023, mockLeague2023_fake2],
            [mockLeague2022_fake1, mockLeague2022, mockLeague2022_fake2],
        ]
        mockLoginProcess = mockMultiprocessingProcess.return_value
        mockLoginProcess.is_alive.return_value = False  # Simulate login process completion

        # load league
        yahooLeagueLoader = YahooLeagueLoader(
            "456",
            [2022, 2023],
            clientId="cid",
            clientSecret="cs",
            ownerNamesAndAliases={
                "o1": ["Owner 1"],
                "o2": ["Owner 2"],
                "o3": ["Owner 3"],
                "o4": ["Owner 4"],
                "o5": ["Owner 5"],
                "o6": ["Owner 6"],
                "o7": ["Owner 7"],
                "o8": ["Owner 8"],
            },
        )
        league = yahooLeagueLoader.loadLeague()

        # expected league
        team1_2022 = Team(ownerId=1, name="Team 1")
        team2_2022 = Team(ownerId=2, name="Team 2")
        team3_2022 = Team(ownerId=3, name="Team 3")
        team4_2022 = Team(ownerId=4, name="Team 4")
        team5_2022 = Team(ownerId=5, name="Team 5")
        team6_2022 = Team(ownerId=6, name="Team 6")
        team7_2022 = Team(ownerId=7, name="Team 7")
        team8_2022 = Team(ownerId=8, name="Team 8")

        team1_2023 = Team(ownerId=1, name="Team 1")
        team2_2023 = Team(ownerId=2, name="Team 2")
        team3_2023 = Team(ownerId=3, name="Team 3")
        team4_2023 = Team(ownerId=4, name="Team 4")
        team5_2023 = Team(ownerId=5, name="Team 5")
        team6_2023 = Team(ownerId=6, name="Team 6")
        team7_2023 = Team(ownerId=7, name="Team 7")
        team8_2023 = Team(ownerId=8, name="Team 8")

        expectedLeague = League(
            name="Test League 2023",
            owners=[
                Owner(name="o1"),
                Owner(name="o2"),
                Owner(name="o3"),
                Owner(name="o4"),
                Owner(name="o5"),
                Owner(name="o6"),
                Owner(name="o7"),
                Owner(name="o8"),
            ],
            years=[
                Year(
                    yearNumber=2022,
                    teams=[
                        team1_2022,
                        team2_2022,
                        team3_2022,
                        team4_2022,
                        team5_2022,
                        team6_2022,
                        team7_2022,
                        team8_2022,
                    ],
                    weeks=[
                        Week(
                            weekNumber=1,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team2_2022.id,
                                    teamAScore=100,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2022.id,
                                    teamBId=team4_2022.id,
                                    teamAScore=100.1,
                                    teamBScore=90.1,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2022.id,
                                    teamBId=team6_2022.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team7_2022.id,
                                    teamBId=team8_2022.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=2,
                            matchups=[
                                Matchup(
                                    teamAId=team2_2022.id,
                                    teamBId=team3_2022.id,
                                    teamAScore=101,
                                    teamBScore=91,
                                    matchupType=MatchupType.PLAYOFF,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                )
                            ],
                        ),
                        Week(
                            weekNumber=3,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2022.id,
                                    teamBId=team2_2022.id,
                                    teamAScore=102,
                                    teamBScore=92,
                                    matchupType=MatchupType.CHAMPIONSHIP,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                )
                            ],
                        ),
                    ],
                    yearSettings=None,
                ),
                Year(
                    yearNumber=2023,
                    teams=[
                        team1_2023,
                        team2_2023,
                        team3_2023,
                        team4_2023,
                        team5_2023,
                        team6_2023,
                        team7_2023,
                        team8_2023,
                    ],
                    weeks=[
                        Week(
                            weekNumber=1,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2023.id,
                                    teamBId=team2_2023.id,
                                    teamAScore=100,
                                    teamBScore=100,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team3_2023.id,
                                    teamBId=team4_2023.id,
                                    teamAScore=100.1,
                                    teamBScore=90.1,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team5_2023.id,
                                    teamBId=team6_2023.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                                Matchup(
                                    teamAId=team7_2023.id,
                                    teamBId=team8_2023.id,
                                    teamAScore=100,
                                    teamBScore=90,
                                    matchupType=MatchupType.REGULAR_SEASON,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                ),
                            ],
                        ),
                        Week(
                            weekNumber=2,
                            matchups=[
                                Matchup(
                                    teamAId=team2_2023.id,
                                    teamBId=team3_2023.id,
                                    teamAScore=101,
                                    teamBScore=91,
                                    matchupType=MatchupType.PLAYOFF,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                )
                            ],
                        ),
                        Week(
                            weekNumber=3,
                            matchups=[
                                Matchup(
                                    teamAId=team1_2023.id,
                                    teamBId=team2_2023.id,
                                    teamAScore=102,
                                    teamBScore=92,
                                    matchupType=MatchupType.CHAMPIONSHIP,
                                    teamAHasTiebreaker=True,
                                    teamBHasTiebreaker=False,
                                )
                            ],
                        ),
                    ],
                    yearSettings=None,
                ),
            ],
        )

        # make sure we called login correctly
        mockMultiprocessingProcess.assert_called_once_with(
            target=yahooLeagueLoader.login, args=("cid", "cs")
        )

        self.assertTrue(league.equals(expectedLeague, ignoreBaseIds=True, ignoreIds=True))
        # check multiWeekMatchupIds
        for year in league.years:
            for week in year.weeks:
                for matchup in week.matchups:
                    self.assertIsNone(matchup.multiWeekMatchupId)

    @mock.patch("multiprocessing.Process")
    @mock.patch("yahoofantasy.Context.__init__")
    @mock.patch("yahoofantasy.Context.get_leagues")
    def test_loadLeague_happyPath(
        self, mockYahooContextGetLeagues, mockYahooContextInit, mockMultiprocessingProcess
    ):
        # mock real league 2022
        mockLeague2022 = Mock()
        mockLeague2022.name = "Test League 2022"
        mockLeague2022.league_id = "123"
        mockLeague2022.season = 2022
        mockLeague2022.current_week = 3
        mockLeague2022.end_week = 3

        # mock fake leagues 2022
        mockLeague2022_fake1 = Mock()
        mockLeague2022_fake1.league_id = "456"

        mockLeague2022_fake2 = Mock()
        mockLeague2022_fake2.league_id = "999"

        # mock league 2023
        mockLeague2023 = Mock()
        mockLeague2023.name = "Test League 2023"
        mockLeague2023.league_id = "456"
        mockLeague2023.season = 2023
        mockLeague2023.current_week = 3
        mockLeague2023.end_week = 3
        mockLeague2023.past_league_id = ["", "123"]

        # mock fake leagues 2023
        mockLeague2023_fake1 = Mock()
        mockLeague2023_fake1.league_id = "123"

        mockLeague2023_fake2 = Mock()
        mockLeague2023_fake2.league_id = "999"

        # mock teams 2022
        mockYahooTeam1_2022 = self.__getMockYahooTeam(
            teamId=1, teamKey=1, name="Team 1", managerNickname="Owner 1", managerId=1
        )
        mockYahooTeam2_2022 = self.__getMockYahooTeam(
            teamId=2, teamKey=2, name="Team 2", managerNickname="Owner 2", managerId=2
        )
        mockYahooTeam3_2022 = self.__getMockYahooTeam(
            teamId=3, teamKey=3, name="Team 3", managerNickname="Owner 3", managerId=3
        )
        mockYahooTeam4_2022 = self.__getMockYahooTeam(
            teamId=4, teamKey=4, name="Team 4", managerNickname="Owner 4", managerId=4
        )
        mockYahooTeam5_2022 = self.__getMockYahooTeam(
            teamId=5, teamKey=5, name="Team 5", managerNickname="Owner 5", managerId=5
        )
        mockYahooTeam6_2022 = self.__getMockYahooTeam(
            teamId=6, teamKey=6, name="Team 6", managerNickname="Owner 6", managerId=6
        )
        mockYahooTeam7_2022 = self.__getMockYahooTeam(
            teamId=7, teamKey=7, name="Team 7", managerNickname="Owner 7", managerId=7
        )
        mockYahooTeam8_2022 = self.__getMockYahooTeam(
            teamId=8, teamKey=8, name="Team 8", managerNickname="Owner 8", managerId=8
        )
        mockLeague2022.teams = self.__getMockTeamsMethod(
            [
                mockYahooTeam1_2022,
                mockYahooTeam2_2022,
                mockYahooTeam3_2022,
                mockYahooTeam4_2022,
                mockYahooTeam5_2022,
                mockYahooTeam6_2022,
                mockYahooTeam7_2022,
                mockYahooTeam8_2022,
            ]
        )

        # mock teams 2023
        mockYahooTeam1_2023 = self.__getMockYahooTeam(
            teamId=1, teamKey=1, name="Team 1", managerNickname="Owner 1", managerId=1
        )
        mockYahooTeam2_2023 = self.__getMockYahooTeam(
            teamId=2, teamKey=2, name="Team 2", managerNickname="Owner 2", managerId=2
        )
        mockYahooTeam3_2023 = self.__getMockYahooTeam(
            teamId=3, teamKey=3, name="Team 3", managerNickname="Owner 3", managerId=3
        )
        mockYahooTeam4_2023 = self.__getMockYahooTeam(
            teamId=4, teamKey=4, name="Team 4", managerNickname="Owner 4", managerId=4
        )
        mockYahooTeam5_2023 = self.__getMockYahooTeam(
            teamId=5, teamKey=5, name="Team 5", managerNickname="Owner 5", managerId=5
        )
        mockYahooTeam6_2023 = self.__getMockYahooTeam(
            teamId=6, teamKey=6, name="Team 6", managerNickname="Owner 6", managerId=6
        )
        mockYahooTeam7_2023 = self.__getMockYahooTeam(
            teamId=7, teamKey=7, name="Team 7", managerNickname="Owner 7", managerId=7
        )
        mockYahooTeam8_2023 = self.__getMockYahooTeam(
            teamId=8, teamKey=8, name="Team 8", managerNickname="Owner 8", managerId=8
        )
        mockLeague2023.teams = self.__getMockTeamsMethod(
            [
                mockYahooTeam1_2023,
                mockYahooTeam2_2023,
                mockYahooTeam3_2023,
                mockYahooTeam4_2023,
                mockYahooTeam5_2023,
                mockYahooTeam6_2023,
                mockYahooTeam7_2023,
                mockYahooTeam8_2023,
            ]
        )

        # mock matchups 2022
        # week -> matchup number -> year
        mockYahooMatchup1_1_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=100),
        ]
        mockYahooMatchup1_2_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=3,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_2_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2022, teamPointsTotal=100.1),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam4_2022, teamPointsTotal=90.1),
        ]
        mockYahooMatchup1_3_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=5,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_3_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam5_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam6_2022, teamPointsTotal=90),
        ]
        mockYahooMatchup1_4_2022 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=7,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup1_4_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam7_2022, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam8_2022, teamPointsTotal=90),
        ]
        # playoffs
        mockYahooMatchup2_1_2022 = self.__getMockYahooMatchup(
            week=2,
            status="postevent",
            winnerTeamKey=2,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup2_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=101),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2022, teamPointsTotal=91),
        ]
        # championship
        mockYahooMatchup3_1_2022 = self.__getMockYahooMatchup(
            week=3,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2022,
        )
        mockYahooMatchup3_1_2022.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2022, teamPointsTotal=102),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2022, teamPointsTotal=92),
        ]

        # mock matchups 2023
        # week -> matchup number -> year
        mockYahooMatchup1_1_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=100),
        ]
        mockYahooMatchup1_2_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=3,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_2_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2023, teamPointsTotal=100.1),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam4_2023, teamPointsTotal=90.1),
        ]
        mockYahooMatchup1_3_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=5,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_3_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam5_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam6_2023, teamPointsTotal=90),
        ]
        mockYahooMatchup1_4_2023 = self.__getMockYahooMatchup(
            week=1,
            status="postevent",
            winnerTeamKey=7,
            isPlayoffs=0,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup1_4_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam7_2023, teamPointsTotal=100),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam8_2023, teamPointsTotal=90),
        ]
        # playoffs
        mockYahooMatchup2_1_2023 = self.__getMockYahooMatchup(
            week=2,
            status="postevent",
            winnerTeamKey=2,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup2_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=101),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam3_2023, teamPointsTotal=91),
        ]
        # championship
        mockYahooMatchup3_1_2023 = self.__getMockYahooMatchup(
            week=3,
            status="postevent",
            winnerTeamKey=1,
            isPlayoffs=1,
            isTied=0,
            isConsolation=0,
            league=mockLeague2023,
        )
        mockYahooMatchup3_1_2023.teams.team = [
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam1_2023, teamPointsTotal=102),
            self.__setMockYahooTeamPoints(mockYahooTeam=mockYahooTeam2_2023, teamPointsTotal=92),
        ]

        # mock weeks 2022
        mockWeek1_2022 = Mock()
        mockWeek1_2022.matchups = [
            mockYahooMatchup1_1_2022,
            mockYahooMatchup1_2_2022,
            mockYahooMatchup1_3_2022,
            mockYahooMatchup1_4_2022,
        ]

        mockWeek2_2022 = Mock()
        mockWeek2_2022.matchups = [mockYahooMatchup2_1_2022]

        mockWeek3_2022 = Mock()
        mockWeek3_2022.matchups = [mockYahooMatchup3_1_2022]

        mockLeague2022.weeks = self.__getMockWeeksMethod(
            [mockWeek1_2022, mockWeek2_2022, mockWeek3_2022]
        )

        # mock weeks 2023
        mockWeek1_2023 = Mock()
        mockWeek1_2023.matchups = [
            mockYahooMatchup1_1_2023,
            mockYahooMatchup1_2_2023,
            mockYahooMatchup1_3_2023,
            mockYahooMatchup1_4_2023,
        ]

        mockWeek2_2023 = Mock()
        mockWeek2_2023.matchups = [mockYahooMatchup2_1_2023]

        mockWeek3_2023 = Mock()
        mockWeek3_2023.matchups = [mockYahooMatchup3_1_2023]

        mockLeague2023.weeks = self.__getMockWeeksMethod(
            [mockWeek1_2023, mockWeek2_2023, mockWeek3_2023]
        )

        mockYahooContextInit.side_effect = [None]
        mockYahooContextGetLeagues.side_effect = [
            [mockLeague2023_fake1, mockLeague2023, mockLeague2023_fake2],
            [mockLeague2022_fake1, mockLeague2022, mockLeague2022_fake2],
        ]
        mockLoginProcess = mockMultiprocessingProcess.return_value
        mockLoginProcess.is_alive.return_value = False  # Simulate login process completion

        # load league
        yahooLeagueLoader = YahooLeagueLoader(
            "456", [2022, 2023], clientId="cid", clientSecret="cs", leagueName="custom name"
        )
        league = yahooLeagueLoader.loadLeague()

        self.assertEqual("custom name", league.name)



import unittest

from leeger.model.league.Team import Team


class TestTeam(unittest.TestCase):
    def test_team_init(self):
        team = Team(ownerId="ownerId", name="name", divisionId="did")

        self.assertEqual("ownerId", team.ownerId)
        self.assertEqual("name", team.name)
        self.assertEqual("did", team.divisionId)

    def test_team_eq_callsEqualsMethod(self):
        # create Team 1
        team_1 = Team(ownerId="1", name="1", divisionId="did")

        # create Team 2
        team_2 = Team(ownerId="1", name="1", divisionId="did")

        team_2.id = team_1.id

        result = team_1 == team_2

        self.assertTrue(result)

    def test_team_eq_equal(self):
        # create Team 1
        team_1 = Team(ownerId="1", name="1", divisionId="did")

        # create Team 2
        team_2 = Team(ownerId="1", name="1", divisionId="did")

        self.assertTrue(team_1.equals(team_2, ignoreBaseIds=True))

    def test_team_eq_notEqual(self):
        # create Team 1
        team_1 = Team(ownerId="1", name="1", divisionId="did")

        # create Team 2
        team_2 = Team(ownerId="2", name="2dif", divisionId="did")

        self.assertFalse(team_1.equals(team_2, ignoreBaseIds=True))

    def test_team_toFromJson(self):
        team = Team(ownerId="", name="team", divisionId="did")

        self.assertEqual(team, Team.fromJson(team.toJson()))



import multiprocessing
import subprocess
from typing import Optional

from yahoofantasy import Context as YahooContext
from yahoofantasy import League as YahooLeague
from yahoofantasy import Matchup as YahooMatchup
from yahoofantasy import Team as YahooTeam
from yahoofantasy import Week as YahooWeek

from leeger.enum.MatchupType import MatchupType
from leeger.exception.DoesNotExistException import DoesNotExistException
from leeger.exception.LeagueLoaderException import LeagueLoaderException
from leeger.league_loader.LeagueLoader import LeagueLoader
from leeger.model.league.League import League
from leeger.model.league.Matchup import Matchup
from leeger.model.league.Owner import Owner
from leeger.model.league.Team import Team
from leeger.model.league.Week import Week
from leeger.model.league.Year import Year
from leeger.validate import leagueValidation


class YahooLeagueLoader(LeagueLoader):
    """
    Responsible for loading a League from Yahoo Fantasy Football.
    https://football.fantasysports.yahoo.com/
    """

    __NFL = "nfl"

    def __init__(
        self,
        mostRecentLeagueId: str,
        years: list[int],
        *,
        clientId: str,
        clientSecret: str,
        loginTimeoutSeconds: Optional[int] = 20,
        ownerNamesAndAliases: Optional[dict[str, list[str]]] = None,
        leagueName: Optional[str] = None,
    ):
        # validation
        try:
            int(mostRecentLeagueId)
        except ValueError:
            raise ValueError(f"League ID '{mostRecentLeagueId}' could not be turned into an int.")
        super().__init__(
            mostRecentLeagueId,
            years,
            ownerNamesAndAliases=ownerNamesAndAliases,
            leagueName=leagueName,
        )
        self.__clientId = clientId
        self.__clientSecret = clientSecret
        self.__loginTimeoutSeconds = loginTimeoutSeconds
        self.__yahooManagerIdToOwnerMap: dict[int, Owner] = dict()
        self.__yahooTeamIdToTeamMap: dict[str, Team] = dict()
        self.__yearToTeamIdHasLostInPlayoffs: dict[int, dict[int, bool]] = dict()

    def login(self, clientId: str, clientSecret: str) -> None:
        """
        Logs in via Yahoo OAuth.
        Will open up a browser window.
        """
        CLIENT_ID_OPTION = "--client-id"
        CLIENT_SECRET_OPTION = "--client-secret"
        subprocess.call(
            [
                "yahoofantasy",
                "login",
                CLIENT_ID_OPTION,
                clientId,
                CLIENT_SECRET_OPTION,
                clientSecret,
            ]
        )

    def __getAllLeagues(self) -> list[YahooLeague]:
        loginProcess = multiprocessing.Process(
            target=self.login, args=(self.__clientId, self.__clientSecret)
        )
        loginProcess.start()
        loginProcess.join(self.__loginTimeoutSeconds)
        if loginProcess.is_alive():
            loginProcess.terminate()
            raise TimeoutError("Login to yahoofantasy timed out.")
        yahooContext = YahooContext()
        yahooLeagues = list()
        # years from most -> least recent
        remainingYears = sorted(self._years, reverse=True)
        currentLeagueId = self._leagueId
        previousLeagueId = None
        for year in remainingYears:
            foundLeagueForYear = False
            # get all leagues this user was in for this year
            leagues = yahooContext.get_leagues(self.__NFL, year)
            # find the league that we want (has a matching ID)
            for league in leagues:
                if str(league.league_id) == currentLeagueId:
                    yahooLeagues.append(league)
                    foundLeagueForYear = True
                    try:
                        # NOTE: this is weird. apparently the league ID here is returned like this: (414, 957486)
                        # NOTE: and the league id is always at index 1
                        previousLeagueId = str(league.past_league_id[1])
                    except Exception as e:
                        self._LOGGER.warning(f"could not get previous league id: {e}")
            if not foundLeagueForYear:
                raise LeagueLoaderException(
                    f"Could not find league for year {year} with ID {currentLeagueId}."
                )
            currentLeagueId = previousLeagueId

        self._validateRetrievedLeagues(yahooLeagues)
        return yahooLeagues

    def getOwnerNames(self) -> dict[int, list[str]]:
        yearToOwnerNamesMap: dict[int, list[str]] = dict()
        yahooLeagues = self.__getAllLeagues()
        for yahooLeague in yahooLeagues:
            yearToOwnerNamesMap[yahooLeague.season] = list()
            yahooTeams = yahooLeague.teams()
            for yahooTeam in yahooTeams:
                ownerName = yahooTeam.manager.nickname
                yearToOwnerNamesMap[yahooLeague.season].append(ownerName)
        return yearToOwnerNamesMap

    def loadLeague(self, validate: bool = True) -> League:
        yahooLeagues = self.__getAllLeagues()
        league = self.__buildLeague(yahooLeagues)
        if validate:
            # validate new league
            leagueValidation.runAllChecks(league)
        self._warnForUnusedOwnerNames(league)
        return league

    def __buildLeague(self, yahooLeagues: list[YahooLeague]) -> League:
        years = list()
        for yahooLeague in yahooLeagues:
            # save league name for each year
            self._leagueNameByYear[yahooLeague.season] = yahooLeague.name
            self.__loadOwners(yahooLeague.teams())
            years.append(self.__buildYear(yahooLeague))
        return League(
            name=self._getLeagueName(),
            owners=list(self.__yahooManagerIdToOwnerMap.values()),
            years=self._getValidYears(years),
        )

    def __buildYear(self, yahooLeague: YahooLeague) -> Year:
        self.__yearToTeamIdHasLostInPlayoffs[yahooLeague.season] = dict()
        teams = self.__buildTeams(yahooLeague.teams())
        weeks = self.__buildWeeks(yahooLeague)
        return Year(yearNumber=yahooLeague.season, teams=teams, weeks=weeks)

    def __buildWeeks(self, yahooLeague: YahooLeague) -> list[Week]:
        weeks = list()
        for i in range(
            yahooLeague.current_week
        ):  # current week seems to be the last week in the league
            yahooWeek: YahooWeek = yahooLeague.weeks()[i]
            # get each teams matchup for that week
            matchups = list()
            # only get matchups that are completed
            validYahooMatchups = [m for m in yahooWeek.matchups if m.status == "postevent"]
            for yahooMatchup in validYahooMatchups:
                # team A is *this* team
                yahooTeamA = yahooMatchup.teams.team[0]
                teamA = self.__yahooTeamIdToTeamMap[yahooTeamA.team_id]
                teamAScore = yahooTeamA.team_points.total
                # team B is their opponent
                yahooTeamB = yahooMatchup.teams.team[1]
                teamB = self.__yahooTeamIdToTeamMap[yahooTeamB.team_id]
                teamBScore = yahooTeamB.team_points.total
                # figure out tiebreakers if there needs to be one
                teamAHasTiebreaker = False
                teamBHasTiebreaker = False
                if yahooMatchup.is_tied == 0:
                    # non-tied matchup
                    teamAHasTiebreaker = yahooMatchup.winner_team_key == yahooTeamA.team_key
                    teamBHasTiebreaker = yahooMatchup.winner_team_key == yahooTeamB.team_key
                matchupType = self.__getMatchupType(yahooMatchup)
                matchups.append(
                    Matchup(
                        teamAId=teamA.id,
                        teamBId=teamB.id,
                        teamAScore=teamAScore,
                        teamBScore=teamBScore,
                        teamAHasTiebreaker=teamAHasTiebreaker,
                        teamBHasTiebreaker=teamBHasTiebreaker,
                        matchupType=matchupType,
                    )
                )
            if len(matchups) > 0:
                weeks.append(Week(weekNumber=i + 1, matchups=matchups))
        return weeks

    def __getMatchupType(self, yahooMatchup: YahooMatchup) -> MatchupType:
        team1Id = yahooMatchup.teams.team[0].team_id
        team2Id = yahooMatchup.teams.team[1].team_id
        # check if this is a playoff week
        if yahooMatchup.is_playoffs == 1:
            # figure out if this is the last week of playoffs (the championship week)
            if yahooMatchup.week == yahooMatchup.league.end_week:
                # this is the championship week
                # figure out if either team has lost in the playoffs yet (or hasn't played in the playoffs yet, in case their first game is the championship)
                if (
                    (
                        team1Id
                        not in self.__yearToTeamIdHasLostInPlayoffs[yahooMatchup.league.season]
                        or not self.__yearToTeamIdHasLostInPlayoffs[yahooMatchup.league.season][
                            team1Id
                        ]
                    )
                    and (
                        team2Id
                        not in self.__yearToTeamIdHasLostInPlayoffs[yahooMatchup.league.season]
                        or not self.__yearToTeamIdHasLostInPlayoffs[yahooMatchup.league.season][
                            team2Id
                        ]
                    )
                    and yahooMatchup.is_consolation == 0
                ):
                    return MatchupType.CHAMPIONSHIP
            # update tracking dict with the team that lost
            for yahooTeamResult in yahooMatchup.teams.team:
                self.__yearToTeamIdHasLostInPlayoffs[yahooMatchup.league.season][
                    yahooTeamResult.team_id
                ] = (yahooTeamResult.team_key != yahooMatchup.winner_team_key)
            return MatchupType.PLAYOFF
        else:
            return MatchupType.REGULAR_SEASON

    def __buildTeams(self, yahooTeams: list[YahooTeam]) -> list[Team]:
        teams = list()
        for yahooTeam in yahooTeams:
            owner = self.__getOwnerByYahooTeam(yahooTeam)
            team = Team(ownerId=owner.id, name=yahooTeam.name)
            teams.append(team)
            self.__yahooTeamIdToTeamMap[yahooTeam.team_id] = team
        return teams

    def __loadOwners(self, yahooTeams: list[YahooTeam]) -> None:
        if len(self.__yahooManagerIdToOwnerMap.values()) == 0:
            yahooManagerIdToOwnerMap = dict()
            yahooOwnerTeamNames = list()
            for yahooTeam in yahooTeams:
                ownerName = yahooTeam.manager.nickname
                # get general owner name if there is one
                generalOwnerName = self._getGeneralOwnerNameFromGivenOwnerName(ownerName)
                ownerName = generalOwnerName if generalOwnerName is not None else ownerName
                # prevent duplicate owner names
                i = 2
                while ownerName in yahooOwnerTeamNames:
                    ownerName = f"{yahooTeam.manager.nickname}({i})"
                yahooManagerIdToOwnerMap[yahooTeam.manager.manager_id] = Owner(name=ownerName)
                yahooOwnerTeamNames.append(ownerName)
            self.__yahooManagerIdToOwnerMap = yahooManagerIdToOwnerMap

    def __getOwnerByYahooTeam(self, yahooTeam: YahooTeam) -> Owner:
        yahooManagerId = yahooTeam.manager.manager_id
        if yahooManagerId in self.__yahooManagerIdToOwnerMap:
            return self.__yahooManagerIdToOwnerMap[yahooManagerId]
        raise DoesNotExistException(
            f"Yahoo Manager ID {yahooManagerId} not found in saved Yahoo Manager IDs."
        )
